{Unit to test if a number is prime and to find next or previus prime
Copyright (C) 2005-2010 Franco Milani
e-mail: nadir@solaris.polarhome.com or astlab@inwind.it
This unit is part of the bigint package v. 1.8 (bigint.zip) available at:
http://www.polarhome.com/~franco/
http://www.polarhome.com:723/~nadir/software/
http://spazioinwind.libero.it/frm/software/
-------------------------------------------------------------------------
This software is released in the hope that it will be useful, but on
an as-is basis. No warranty of any kind is offered, either expressed
or implied, including, without limitation, the implied warranties of
merchantability and fitness for a particular purpose.
You can use or modify this unit for personal purposes. You can also
redistribute it, but only in the original unmodified form and keeping
intact any part, included the above copyright notice. Possible changes
in the distributed versions will be made only by the author according
to hints, requests or bug reports.
You may distribute software items that link this unit or part of it
provided that you inform the recipients that this unit is employed.
-------------------------------------------------------------------------
The function is_prime(N) returns 1 if the number N is prime or 0 otherwise.
If N < 2^32 it uses a Miller-Rabin test to bases 2, 7 and 61, a proper
primality test in this range ([1]).
If N > 2^32 it performs first a trial division by the primes up to
997, then a Baillie-PSW test using a variant of Wei Dai (i.e. it tests
if N is a strong pseudoprime to the base 2 and a Lucas pseudoprime).
If the function returns 0 then N is certainly composite. If it returns 1
the result is probabilistic but reliable, in the sense that no composite
number is currently known to pass this test. Nobody has yet claimed the
prize that is offered for a counterexample. It is conjectured that no
such composite numbers exist, and anyhow if they exist, they are extremely
rare and out of the usual ranges.

The procedure next_prime(N,X) returns the smallest prime number greater
than N. The procedure prec_prime(N,X) if N>=3 returns the largest prime
number least than N, if N<3 returns zero. The results are achieved in fast
mode skipping the multiples of 2, 3, 5 and 7 by the remainders modulo 210.

References

[1] G.Jaeschke, "On strong pseudoprimes to several bases", Mathematics of
computation, 61 (1993) 915-926
[2] R. Baillie and Wagstaff, Jr, S. S., "Lucas Pseudoprimes", Mathematics
of computation, 35 (1980) 1391-1417
[3] C. Pomerance, J. L. Selfridge and Wagstaff, Jr, S. S., "The pseudoprimes
to 25*10^9", Mathematics of computation, 35 (1980) 1003-1026
-------------------------------------------------------------------------}
{$packrecords 1}
{$asmmode intel}
{$ifndef ver1_0}{$calling oldfpccall}{$endif}
UNIT isprn;

INTERFACE

USES bigint18;

FUNCTION is_prime (N: nat): longint;
PROCEDURE next_prime (N: nat; var X: nat);
PROCEDURE prec_prime (N: nat; var X: nat);

IMPLEMENTATION

CONST

pin: array [0..61] of byte =
(0,0,1,1,0,1,0,1,0,0,0,1,0,1,0,0,0,1,0,1,0,0,0,1,0,0,0,0,0,1,0,1,0,0,0,0,0,1,
0,0,0,1,0,1,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0,1,0,1);

pmq: array [1..16] of byte =
(61,59,53,47,43,41,37,31,29,23,17,13,11,7,5,3);

aprimes : array[1..168] of word =
(2,3,5,7,11,13,17,19,23,29,31,37,41,43,47,53,59,61,67,71,73,79,83,89,97,101,103,
107,109,113,127,131,137,139,149,151,157,163,167,173,179,181,191,193,197,199,211,
223,227,229,233,239,241,251,257,263,269,271,277,281,283,293,307,311,313,317,331,
337,347,349,353,359,367,373,379,383,389,397,401,409,419,421,431,433,439,443,449,
457,461,463,467,479,487,491,499,503,509,521,523,541,547,557,563,569,571,577,587,
593,599,601,607,613,617,619,631,641,643,647,653,659,661,673,677,683,691,701,709,
719,727,733,739,743,751,757,761,769,773,787,797,809,811,821,823,827,829,839,853,
857,859,863,877,881,883,887,907,911,919,929,937,941,947,953,967,971,977,983,991,
997);

nxtb: array [0..29] of longword =
(2,2,3,5,5,7,7,11,11,11,11,13,13,17,17,17,17,19,19,23,23,23,23,29,29,29,29,29,
29,31);

pxtb: array [0..29] of longword =
(0,0,0,2,3,3,5,5,7,7,7,7,11,11,13,13,13,13,17,17,19,19,19,19,23,23,23,23,23,
23);

indm: array [0..104] of longword =
(0,64,64,64,64,1,2,64,3,4,64,5,64,64,6,7,64,64,8,64,9,10,64,11,64,64,12,64,64,
13,14,64,64,15,64,16,17,64,64,18,64,19,64,64,20,64,64,64,21,64,22,23,64,24,25,
64,26,64,64,64,27,64,64,28,64,29,64,64,30,31,64,32,64,64,33,34,64,64,35,64,64,
36,64,37,38,64,39,64,64,40,41,64,64,42,64,43,44,64,45,46,64,64,64,64,47);

difm: array [0..47] of longword =
(10,2,4,2,4,6,2,6,4,2,4,6,6,2,6,4,2,6,4,6,8,4,2,4,2,4,8,6,4,6,2,4,6,2,6,6,4,2,
4,6,2,6,4,2,4,2,10,2);

VAR

NA,NB,NT,NU,NV,NY: nat;
d,e,i,k,kc,pm,t,nc,kv,u,h,dimn,tk,tq,ct,cu,cv: longword;
mul0,reg_esi,reg_a,reg_b,reg_c,reg_q,reg_s,reg_u: longword;
la1,la2,loc_a,loc_b,loc_la,loc_lb: longword;

FUNCTION is_prime (N: nat): longint; assembler;
ASM
                mov     edi,[ebp+8]
                cmp     dword ptr [edi],1
                jne     @@990
                mov     ebx,[edi+4]
                cmp     ebx,61
                ja      @@100
                lea     edi,pin
                xor     eax,eax
                mov     al,[edi+ebx]
                jmp     @@100010
@@100:          test    ebx,1
                jz      @@100000
                mov     pm,ebx
                lea     edi,pmq
                mov     esi,15
                xor     ecx,ecx
@@105:          mov     eax,ebx
                xor     edx,edx
                mov     cl,[edi+esi]
                div     ecx
                cmp     edx,0
                je      @@100000
                dec     esi
                jns     @@105
                dec     ebx
                mov     k,ebx
                bsf     ecx,ebx
                mov     e,ecx
                jz      @@150
                dec     e
@@150:          shr     ebx,cl
                mov     d,ebx
                mov     kc,2
                mov     eax,2
                jmp     @@171
@@180:          mov     eax,7
                jmp     @@170
@@190:          mov     eax,61
@@170:          mov     ebx,d
@@171:          mov     ecx,pm
                cmp     ebx,1
                jne     @@200
                xor     edx,edx
                div     ecx
                mov     eax,edx
                jmp     @@210
@@200:          mov     edi,eax
                bsr     edx,ebx
                mov     t,edx
                bt      ebx,0
                jnc     @@220
                mov     esi,eax
                jmp     @@240
@@220:          mov     esi,1
@@240:          mov     i,1
@@250:          mov     eax,edi
                mul     edi
                cmp     edx,ecx
                jb      @@260
                mov     ebx,eax
                mov     eax,edx
                xor     edx,edx
                div     ecx
                mov     eax,ebx
@@260:          div     ecx
                mov     edi,edx
                mov     eax,d
                mov     ebx,i
                bt      eax,ebx
                jnc     @@270
                mov     eax,esi
                mul     edi
                cmp     edx,ecx
                jb      @@280
                mov     ebx,eax
                mov     eax,edx
                xor     edx,edx
                div     ecx
                mov     eax,ebx
@@280:          div     ecx
                mov     esi,edx
@@270:          inc     i
                dec     t
                jnz     @@250
                mov     eax,esi
@@210:          cmp     eax,1
                je      @@290
                cmp     eax,k
                je      @@290
                cmp     e,0
                je      @@100000
                mov     esi,e
@@300:          mul     eax
                cmp     edx,ecx
                jb      @@310
                mov     ebx,eax
                mov     eax,edx
                xor     edx,edx
                div     ecx
                mov     eax,ebx
@@310:          div     ecx
                mov     eax,edx
                cmp     eax,k
                je      @@290
                cmp     eax,1
                je      @@100000
@@320:          dec     esi
                jnz     @@300
                jmp     @@100000
@@290:          dec     kc
                jz      @@190
                jns     @@180
                mov     eax,1
                jmp     @@100010
@@990:          test    dword ptr [edi+4],1
                jz      @@100000
                lea     esi,aprimes
                mov     t,167
                mov     eax,[edi]
                mov     dimn,eax
                xor     ebx,ebx
@@992:          inc     esi
                inc     esi
                mov     bx,[esi]
                xor     edx,edx
                mov     ecx,dimn
@@995:          mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@995
                cmp     edx,0
                je      @@100000
                dec     t
                jnz     @@992
                mov     edi,[ebp+8]
                lea     esi,NU
                mov     ecx,[edi]
                mov     [esi],ecx
@@1000:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@1000
                mov     ecx,[edi]
                add     esi,4
                mov     eax,1
                sub     [esi],eax
                jnc     @@1010
                dec     ecx
@@1020:         add     esi,4
                sbb     dword ptr[esi],1
                jnc     @@1030
                dec     ecx
                jnz     @@1020
                jmp     @@1010
@@1030:         jnz     @@1010
                dec     ecx
                jnz     @@1010
                lea     esi,NU
                dec     dword ptr [esi]
@@1010:         lea     esi,NU
                mov     ecx,[esi]
                xor     edx,edx
                lea     esi,[esi+ecx*4+4]
                neg     ecx
                mov     eax,[esi+ecx*4]
                cmp     eax,0
                jne     @@1060
                inc     ecx
                jz      @@1080
                mov     edx,32
@@1040:         mov     eax,[esi+ecx*4]
                cmp     eax,0
                jne     @@1060
                add     edx,32
                inc     ecx
                jnz     @@1040
                xor     edx,edx
                jmp     @@1080
@@1060:         bsf     ecx,eax
                add     edx,ecx
@@1080:         mov     e,edx
                lea     edi,NU
                lea     esi,NY
                mov     ecx,edx
                mov     edx,[edi]
                cmp     ecx,0
                jne     @@1100
                mov     [esi],edx
@@1110:         mov     eax,[edi+edx*4]
                mov     [esi+edx*4],eax
                dec     edx
                jnz     @@1110
                jmp     @@1120
@@1100:         cmp     ecx,32
                jb      @@1130
                mov     eax,ecx
                shr     eax,5
                cmp     eax,edx
                jb      @@1140
                mov     dword ptr [esi],1
                mov     dword ptr [esi+4],0
                jmp     @@1120
@@1140:         mov     ebx,eax
                shl     ebx,5
                sub     ecx,ebx
                jnz     @@1150
                mov     ecx,edx
                sub     ecx,eax
                mov     [esi],ecx
@@1160:         mov     eax,[edi+edx*4]
                mov     [esi+ecx*4],eax
                dec     edx
                dec     ecx
                jnz     @@1160
                jmp     @@1120
@@1150:         sub     edx,eax
                lea     edi,[edi+eax*4]
@@1130:         mov     [esi],edx
                lea     edi,[edi+edx*4+4]
                lea     esi,[esi+edx*4]
                neg     edx
                mov     ebx,[edi+edx*4]
                inc     edx
                jnz     @@1170
                shr     ebx,cl
                mov     [esi],ebx
                jmp     @@1120
@@1170:         mov     eax,[edi+edx*4]
                shrd    ebx,eax,cl
                mov     [esi+edx*4],ebx
                mov     ebx,eax
                inc     edx
                jnz     @@1170
                shr     ebx,cl
                mov     [esi],ebx
                lea     esi,NY
                mov     ecx,[esi]
@@1180:         cmp     dword ptr [esi+ecx*4],0
                jne     @@1190
                dec     ecx
                jnz     @@1180
                inc     ecx
@@1190:         mov     [esi],ecx
@@1120:         lea     edi,NT
                mov     dword ptr [edi],1
                mov     dword ptr [edi+4],2
                lea     edx,NY
                lea     ebx,NV
                lea     esi,NA
                mov     ecx,[edi]
                mov     [esi],ecx
                test    dword ptr [edx+4],1
                jz      @@2000
                mov     [ebx],ecx
@@2010:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                mov     [ebx+ecx*4],eax
                dec     ecx
                jnz     @@2010
                jmp     @@2020
@@2000:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@2000
                mov     dword ptr [ebx],1
                mov     dword ptr [ebx+4],1
@@2020:         mov     ecx,[edx]
                mov     eax,[edx+ecx*4]
                cmp     eax,0
                je      @@2030
                bsr     eax,eax
                inc     eax
@@2030:         dec     ecx
                shl     ecx,5
                add     eax,ecx
                dec     eax
                jz      @@2040
                mov     nc,eax
                mov     kv,0
@@2050:         inc     kv
                lea     esi,NA
                mov     ecx,[esi]
                mov     edi,esi
                mov     edx,ecx
                lea     ebx,NB
                mov     pm,ecx
                add     pm,ecx
                add     esi,4
                add     edi,4
                add     ebx,4
                lea     esi,dword ptr [esi+ecx*4]
                lea     ebx,dword ptr [ebx+ecx*4]
                neg     ecx
                mov     eax,[edi]
                mov     mul0,eax
                mov     loc_lb,edx
                mov     loc_la,ecx
                mov     loc_b,edi
                mov     loc_a,esi
                xor     edi,edi
@@2060:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                mov     [ebx+ecx*4],eax
                mov     edi,edx
                inc     ecx
                jne     @@2060
                mov     [ebx],edx
                dec     loc_lb
                jz      @@2070
@@2080:         add     loc_b,4
                mov     edi,loc_b
                mov     edi,[edi]
                mov     mul0,edi
                mov     esi,loc_a
                add     ebx,4
                mov     ecx,loc_la
                xor     edi,edi
@@2090:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                add     [ebx+ecx*4],eax
                adc     edx,0
                mov     edi,edx
                inc     ecx
                jne     @@2090
                mov     [ebx],edx
                dec     loc_lb
                jnz     @@2080
@@2070:         lea     edi,NB
                mov     ecx,pm
@@2100:         cmp     dword ptr[edi+ecx*4],0
                jne     @@2110
                dec     ecx
                jnz     @@2100
                inc     ecx
@@2110:         mov     [edi],ecx
                mov     la1,ecx
                mov     esi,[ebp+8]
                mov     edx,[esi]
                cmp     edx,la1
                jbe     @@2120
                lea     esi,NA
                mov     [esi],ecx
@@2130:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@2130
                jmp     @@2140
@@2120:         mov     la2,edx
                cmp     edx,2
                ja      @@2150
                je      @@2160
                mov     ebx,[esi+4]
                xor     edx,edx
@@2170:         mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@2170
                lea     ebx,NA
                mov     dword ptr [ebx],1
                mov     [ebx+4],edx
                jmp     @@2140
@@2160:         mov     ebx,[esi+4]
                mov     eax,[esi+8]
                add     edi,4
                xor     edx,edx
                dec     ecx
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@2180:         cmp     edx,reg_b
                je      @@2190
                div     reg_b
                jmp     @@2200
@@2190:         add     edx,eax
                mov     eax,4294967295
                jnc     @@2200
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2210
                mov     ebx,[edi]
@@2210:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@2220
@@2200:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2230
                mov     ebx,[edi]
@@2230:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@2240
                ja      @@2250
                test    eax,eax
                jz      @@2240
@@2250:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@2240
                ja      @@2250
                test    eax,eax
                jnz     @@2250
@@2240:         neg     eax
                adc     edx,0
                neg     edx
@@2220:         popf
                jnz     @@2180
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,NA
                mov     [edi+4],eax
                cmp     edx,0
                jne     @@2260
                mov     dword ptr [edi],1
                jmp     @@2140
@@2260:         mov     [edi+8],edx
                mov     dword ptr [edi],2
                jmp     @@2140
@@2150:         mov     edx,la1
                lea     esi,NA
                mov     [esi],ecx
@@2270:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@2270
@@2280:         lea     esi,[esi+edx*4+4]
                mov     ecx,la2
                sub     edx,ecx
                mov     edi,[ebp+8]
                lea     edi,[edi+ecx*4+4]
                mov     eax,[esi-4]
                cmp     eax,[edi-4]
                jb      @@2290
                mov     dword ptr [esi],0
                add     esi,4
                inc     edx
                jmp     @@2300
@@2290:         test    edx,edx
                jz      @@2310
@@2300:         mov     ebx,[edi-8]
                mov     eax,[edi-4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                mov     reg_a,eax
                mov     eax,[edi-12]
                shld    ebx,eax,cl
                mov     reg_b,ebx
                mov     reg_s,ecx
                mov     reg_c,edx
@@2320:         sub     esi,4
                mov     edx,[esi]
                mov     ebx,[esi-4]
                mov     eax,[esi-8]
                mov     ecx,reg_s
                shld    edx,ebx,cl
                shld    ebx,eax,cl
                mov     reg_u,ebx
                mov     ebx,[esi-12]
                shld    eax,ebx,cl
                mov     ebx,eax
                mov     eax,reg_u
                cmp     reg_a,edx
                je      @@2330
                div     reg_a
                mov     reg_q,eax
                jmp     @@2340
@@2330:         add     edx,eax
                mov     reg_q,4294967295
                jc      @@2350
@@2340:         mov     ecx,edx
                mov     eax,reg_b
                mul     reg_q
                sub     ebx,eax
                sbb     ecx,edx
                jnb     @@2350
                dec     reg_q
                add     ebx,reg_b
                adc     ecx,reg_a
                jc      @@2350
                dec     reg_q
@@2350:         mov     ecx,la2
                neg     ecx
                xor     ebx,ebx
@@2360:         mov     eax,[edi+ecx*4]
                mul     reg_q
                add     eax,ebx
                adc     edx,0
                sub     [esi+ecx*4],eax
                adc     edx,0
                mov     ebx,edx
                inc     ecx
                jne     @@2360
                cmp     [esi],ebx
                je      @@2370
                dec     reg_q
                mov     ecx,la2
                neg     ecx
                clc
@@2380:         mov     eax,[edi+ecx*4]
                adc     [esi+ecx*4],eax
                inc     ecx
                jne     @@2380
@@2370:         dec     reg_c
                jne     @@2320
@@2310:         lea     esi,NA
                mov     ecx,la2
@@2390:         cmp     dword ptr[esi+ecx*4],0
                jne     @@2400
                dec     ecx
                jnz     @@2390
                inc     ecx
@@2400:         mov     [esi],ecx
@@2140:         lea     edi,NY
                mov     edx,kv
                mov     ecx,edx
                and     ecx,31
                shr     edx,5
                inc     edx
                mov     eax,[edi+edx*4]
                bt      eax,ecx
                jnc     @@2410
                lea     esi,NV
                mov     ecx,[esi]
                lea     edi,NA
                mov     edx,[edi]
                lea     ebx,NB
                cmp     ecx,edx
                jae     @@2420
                xchg    edi,esi
                xchg    edx,ecx
@@2420:         mov     pm,ecx
                add     pm,edx
                add     esi,4
                add     edi,4
                add     ebx,4
                lea     esi,dword ptr [esi+ecx*4]
                lea     ebx,dword ptr [ebx+ecx*4]
                neg     ecx
                mov     eax,[edi]
                mov     mul0,eax
                mov     loc_lb,edx
                mov     loc_la,ecx
                mov     loc_b,edi
                mov     loc_a,esi
                xor     edi,edi
@@2430:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                mov     [ebx+ecx*4],eax
                mov     edi,edx
                inc     ecx
                jne     @@2430
                mov     [ebx],edx
                dec     loc_lb
                jz      @@2440
@@2450:         add     loc_b,4
                mov     edi,loc_b
                mov     edi,[edi]
                mov     mul0,edi
                mov     esi,loc_a
                add     ebx,4
                mov     ecx,loc_la
                xor     edi,edi
@@2460:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                add     [ebx+ecx*4],eax
                adc     edx,0
                mov     edi,edx
                inc     ecx
                jne     @@2460
                mov     [ebx],edx
                dec     loc_lb
                jnz     @@2450
@@2440:         lea     edi,NB
                mov     ecx,pm
@@2470:         cmp     dword ptr[edi+ecx*4],0
                jne     @@2480
                dec     ecx
                jnz     @@2470
                inc     ecx
@@2480:         mov     [edi],ecx
                mov     la1,ecx
                mov     esi,[ebp+8]
                mov     edx,[esi]
                cmp     edx,la1
                jbe     @@2490
                lea     esi,NV
                mov     [esi],ecx
@@2500:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@2500
                jmp     @@2410
@@2490:         mov     la2,edx
                cmp     edx,2
                ja      @@2510
                je      @@2520
                mov     ebx,[esi+4]
                xor     edx,edx
@@2530:         mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@2530
                lea     ebx,NV
                mov     dword ptr [ebx],1
                mov     [ebx+4],edx
                jmp     @@2410
@@2520:         mov     ebx,[esi+4]
                mov     eax,[esi+8]
                add     edi,4
                xor     edx,edx
                dec     ecx
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@2540:         cmp     edx,reg_b
                je      @@2550
                div     reg_b
                jmp     @@2560
@@2550:         add     edx,eax
                mov     eax,4294967295
                jnc     @@2560
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2570
                mov     ebx,[edi]
@@2570:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@2580
@@2560:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2590
                mov     ebx,[edi]
@@2590:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@2600
                ja      @@2610
                test    eax,eax
                jz      @@2600
@@2610:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@2600
                ja      @@2610
                test    eax,eax
                jnz     @@2610
@@2600:         neg     eax
                adc     edx,0
                neg     edx
@@2580:         popf
                jnz     @@2540
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,NV
                mov     [edi+4],eax
                cmp     edx,0
                jne     @@2620
                mov     dword ptr [edi],1
                jmp     @@2410
@@2620:         mov     [edi+8],edx
                mov     dword ptr [edi],2
                jmp     @@2410
@@2510:         mov     edx,la1
                lea     esi,NV
                mov     [esi],ecx
@@2630:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@2630
@@2640:         lea     esi,[esi+edx*4+4]
                mov     ecx,la2
                sub     edx,ecx
                mov     edi,[ebp+8]
                lea     edi,[edi+ecx*4+4]
                mov     eax,[esi-4]
                cmp     eax,[edi-4]
                jb      @@2650
                mov     dword ptr [esi],0
                add     esi,4
                inc     edx
                jmp     @@2660
@@2650:         test    edx,edx
                jz      @@2670
@@2660:         mov     ebx,[edi-8]
                mov     eax,[edi-4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                mov     reg_a,eax
                mov     eax,[edi-12]
                shld    ebx,eax,cl
                mov     reg_b,ebx
                mov     reg_s,ecx
                mov     reg_c,edx
@@2680:         sub     esi,4
                mov     edx,[esi]
                mov     ebx,[esi-4]
                mov     eax,[esi-8]
                mov     ecx,reg_s
                shld    edx,ebx,cl
                shld    ebx,eax,cl
                mov     reg_u,ebx
                mov     ebx,[esi-12]
                shld    eax,ebx,cl
                mov     ebx,eax
                mov     eax,reg_u
                cmp     reg_a,edx
                je      @@2690
                div     reg_a
                mov     reg_q,eax
                jmp     @@2700
@@2690:         add     edx,eax
                mov     reg_q,4294967295
                jc      @@2710
@@2700:         mov     ecx,edx
                mov     eax,reg_b
                mul     reg_q
                sub     ebx,eax
                sbb     ecx,edx
                jnb     @@2710
                dec     reg_q
                add     ebx,reg_b
                adc     ecx,reg_a
                jc      @@2710
                dec     reg_q
@@2710:         mov     ecx,la2
                neg     ecx
                xor     ebx,ebx
@@2720:         mov     eax,[edi+ecx*4]
                mul     reg_q
                add     eax,ebx
                adc     edx,0
                sub     [esi+ecx*4],eax
                adc     edx,0
                mov     ebx,edx
                inc     ecx
                jne     @@2720
                cmp     [esi],ebx
                je      @@2730
                dec     reg_q
                mov     ecx,la2
                neg     ecx
                clc
@@2740:         mov     eax,[edi+ecx*4]
                adc     [esi+ecx*4],eax
                inc     ecx
                jne     @@2740
@@2730:         dec     reg_c
                jne     @@2680
@@2670:         lea     esi,NV
                mov     ecx,la2
@@2750:         cmp     dword ptr[esi+ecx*4],0
                jne     @@2760
                dec     ecx
                jnz     @@2750
                inc     ecx
@@2760:         mov     [esi],ecx
@@2410:         dec     nc
                jnz     @@2050
@@2040:         lea     esi,NV
                cmp     dword ptr [esi+4],1
                jne     @@2900
                cmp     dword ptr [esi],1
                je      @@4000
@@2900:         lea     edi,NU
                mov     ecx,[edi]
                cmp     ecx,[esi]
                jne     @@2930
@@2925:         mov     eax,[edi+ecx*4]
                cmp     eax,[esi+ecx*4]
                jne     @@2930
                dec     ecx
                jnz     @@2925
                jmp     @@4000
@@2930:         cmp     e,2
                jb      @@100000
@@2935:         dec     e
@@2936:         mov     ecx,[esi]
                mov     edi,esi
                mov     edx,ecx
                lea     ebx,NA
                mov     pm,ecx
                add     pm,ecx
                add     esi,4
                add     edi,4
                add     ebx,4
                lea     esi,dword ptr [esi+ecx*4]
                lea     ebx,dword ptr [ebx+ecx*4]
                neg     ecx
                mov     eax,[edi]
                mov     mul0,eax
                mov     loc_lb,edx
                mov     loc_la,ecx
                mov     loc_b,edi
                mov     loc_a,esi
                xor     edi,edi
@@3000:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                mov     [ebx+ecx*4],eax
                mov     edi,edx
                inc     ecx
                jne     @@3000
                mov     [ebx],edx
                dec     loc_lb
                jz      @@3010
@@3020:         add     loc_b,4
                mov     edi,loc_b
                mov     edi,[edi]
                mov     mul0,edi
                mov     esi,loc_a
                add     ebx,4
                mov     ecx,loc_la
                xor     edi,edi
@@3030:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                add     [ebx+ecx*4],eax
                adc     edx,0
                mov     edi,edx
                inc     ecx
                jne     @@3030
                mov     [ebx],edx
                dec     loc_lb
                jnz     @@3020
@@3010:         lea     edi,NA
                mov     ecx,pm
@@3040:         cmp     dword ptr[edi+ecx*4],0
                jne     @@3050
                dec     ecx
                jnz     @@3040
                inc     ecx
@@3050:         mov     [edi],ecx
                mov     la1,ecx
                mov     esi,[ebp+8]
                mov     edx,[esi]
                cmp     edx,la1
                jbe     @@3060
                lea     esi,NV
                mov     [esi],ecx
@@3070:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@3070
                jmp     @@3080
@@3060:         mov     la2,edx
                cmp     edx,2
                ja      @@3090
                je      @@3100
                mov     ebx,[esi+4]
                xor     edx,edx
@@3110:         mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@3110
                lea     ebx,NV
                mov     dword ptr [ebx],1
                mov     [ebx+4],edx
                jmp     @@3080
@@3100:         mov     ebx,[esi+4]
                mov     eax,[esi+8]
                add     edi,4
                xor     edx,edx
                dec     ecx
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@3120:         cmp     edx,reg_b
                je      @@3130
                div     reg_b
                jmp     @@3140
@@3130:         add     edx,eax
                mov     eax,4294967295
                jnc     @@3140
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@3150
                mov     ebx,[edi]
@@3150:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@3160
@@3140:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@3170
                mov     ebx,[edi]
@@3170:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@3180
                ja      @@3190
                test    eax,eax
                jz      @@3180
@@3190:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@3180
                ja      @@3190
                test    eax,eax
                jnz     @@3190
@@3180:         neg     eax
                adc     edx,0
                neg     edx
@@3160:         popf
                jnz     @@3120
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,NV
                mov     [edi+4],eax
                cmp     edx,0
                jne     @@3200
                mov     dword ptr [edi],1
                jmp     @@3080
@@3200:         mov     [edi+8],edx
                mov     dword ptr [edi],2
                jmp     @@3080
@@3090:         mov     edx,la1
                lea     esi,NV
                mov     [esi],ecx
@@3210:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@3210
@@3220:         lea     esi,[esi+edx*4+4]
                mov     ecx,la2
                sub     edx,ecx
                mov     edi,[ebp+8]
                lea     edi,[edi+ecx*4+4]
                mov     eax,[esi-4]
                cmp     eax,[edi-4]
                jb      @@3230
                mov     dword ptr [esi],0
                add     esi,4
                inc     edx
                jmp     @@3240
@@3230:         test    edx,edx
                jz      @@3250
@@3240:         mov     ebx,[edi-8]
                mov     eax,[edi-4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                mov     reg_a,eax
                mov     eax,[edi-12]
                shld    ebx,eax,cl
                mov     reg_b,ebx
                mov     reg_s,ecx
                mov     reg_c,edx
@@3260:         sub     esi,4
                mov     edx,[esi]
                mov     ebx,[esi-4]
                mov     eax,[esi-8]
                mov     ecx,reg_s
                shld    edx,ebx,cl
                shld    ebx,eax,cl
                mov     reg_u,ebx
                mov     ebx,[esi-12]
                shld    eax,ebx,cl
                mov     ebx,eax
                mov     eax,reg_u
                cmp     reg_a,edx
                je      @@3270
                div     reg_a
                mov     reg_q,eax
                jmp     @@3280
@@3270:         add     edx,eax
                mov     reg_q,4294967295
                jc      @@3290
@@3280:         mov     ecx,edx
                mov     eax,reg_b
                mul     reg_q
                sub     ebx,eax
                sbb     ecx,edx
                jnb     @@3290
                dec     reg_q
                add     ebx,reg_b
                adc     ecx,reg_a
                jc      @@3290
                dec     reg_q
@@3290:         mov     ecx,la2
                neg     ecx
                xor     ebx,ebx
@@3300:         mov     eax,[edi+ecx*4]
                mul     reg_q
                add     eax,ebx
                adc     edx,0
                sub     [esi+ecx*4],eax
                adc     edx,0
                mov     ebx,edx
                inc     ecx
                jne     @@3300
                cmp     [esi],ebx
                je      @@3310
                dec     reg_q
                mov     ecx,la2
                neg     ecx
                clc
@@3320:         mov     eax,[edi+ecx*4]
                adc     [esi+ecx*4],eax
                inc     ecx
                jne     @@3320
@@3310:         dec     reg_c
                jne     @@3260
@@3250:         lea     esi,NV
                mov     ecx,la2
@@3330:         cmp     dword ptr[esi+ecx*4],0
                jne     @@3340
                dec     ecx
                jnz     @@3330
                inc     ecx
@@3340:         mov     [esi],ecx
@@3080:         lea     edi,NU
                lea     esi,NV
                mov     ecx,[edi]
                cmp     ecx,[esi]
                jne     @@3996
@@3995:         mov     eax,[edi+ecx*4]
                cmp     eax,[esi+ecx*4]
                jne     @@3996
                dec     ecx
                jnz     @@3995
                jmp     @@4000
@@3996:         cmp     dword ptr [esi+4],1
                jne     @@3997
                cmp     dword ptr [esi],1
                je      @@100000
@@3997:         dec     e
                jnz     @@2936
                jmp     @@100000
@@4000:         mov     edi,[ebp+8]
                mov     tq,4294967232
                mov     tk,1
                mov     esi,1
@@4010:         inc     tk
                inc     tk
                inc     tq
                jnz     @@6000
                mov     reg_esi,esi
                lea     esi,NB
                cmp     dword ptr [edi],1
                jne     @@5030
                cmp     dword ptr [edi+4],1
                ja      @@5030
                mov     dword ptr [esi],1
                mov     eax,[edi+4]
                mov     [esi+4],eax
                jmp     @@5040
@@5030:         mov     ecx,[edi]
                mov     eax,[edi+ecx*4]
                cmp     eax,0
                je      @@5050
                bsr     eax,eax
                inc     eax
@@5050:         dec     ecx
                shl     ecx,5
                add     eax,ecx
                mov     edi,eax
                shr     eax,1
                cmp     eax,2
                jae     @@5060
                mov     dword ptr [esi],1
                mov     dword ptr [esi+4],4
                jmp     @@5070
@@5060:         dec     eax
                dec     eax
                lea     esi,NU
                mov     ebx,32
                xor     edx,edx
                div     ebx
                mov     ecx,eax
                inc     eax
                mov     [esi],eax
                jecxz   @@5080
@@5090:         mov     dword ptr[esi+ecx*4],0
                dec     ecx
                jnz     @@5090
@@5080:         mov     ebx,1
                mov     ecx,edx
                shl     ebx,cl
                mov     [esi+eax*4],ebx
                mov     eax,edi
                and     eax,1
                shl     eax,1
                add     eax,5
                lea     esi,NB
                mov     pm,eax
                lea     edi,NU
                mov     ecx,[edi]
                mov     [esi],ecx
                add     edi,4
                add     esi,4
                xor     ebx,ebx
                lea     edi,[edi+ecx*4]
                lea     esi,[esi+ecx*4]
                neg     ecx
@@5100:         mov     eax,[edi+ecx*4]
                mul     pm
                add     eax,ebx
                adc     edx,0
                mov     [esi+ecx*4],eax
                mov     ebx,edx
                inc     ecx
                jne     @@5100
                cmp     ebx,0
                je      @@5070
                lea     esi,NB
                inc     dword ptr [esi]
                mov     ecx,[esi]
                mov     [esi+ecx*4],ebx
@@5070:         mov     edi,[ebp+8]
                mov     ecx,[edi]
                mov     la1,ecx
                lea     esi,NB
                mov     edx,[esi]
                cmp     edx,la1
                jbe     @@5110
                lea     edi,NU
                mov     dword ptr [edi],1
                mov     dword ptr [edi+4],0
                jmp     @@5120
@@5110:         mov     la2,edx
                cmp     edx,2
                ja      @@5130
                je      @@5140
                mov     ebx,[esi+4]
                lea     esi,NU
                xor     edx,edx
@@5150:         mov     eax,[edi+ecx*4]
                div     ebx
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@5150
                mov     ecx,la1
                cmp     dword ptr[esi+ecx*4],0
                jne     @@5160
                dec     ecx
                jnz     @@5160
                inc     ecx
@@5160:         mov     [esi],ecx
                jmp     @@5120
@@5140:         mov     ebx,[esi+4]
                mov     eax,[esi+8]
                lea     esi,NU
                add     edi,4
                add     esi,4
                xor     edx,edx
                dec     ecx
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                lea     esi,[esi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     [esi],edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@5170:         sub     esi,4
                cmp     edx,reg_b
                je      @@5180
                div     reg_b
                jmp     @@5190
@@5180:         add     edx,eax
                mov     eax,4294967295
                jnc     @@5190
                sub     edi,4
                mov     [esi],eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@5200
                mov     ebx,[edi]
@@5200:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@5210
@@5190:         mov     [esi],eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@5220
                mov     ebx,[edi]
@@5220:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     dword ptr [esi]
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@5230
                ja      @@5240
                test    eax,eax
                jz      @@5230
@@5240:         dec     dword ptr [esi]
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@5230
                ja      @@5240
                test    eax,eax
                jnz     @@5240
@@5230:         neg     eax
                adc     edx,0
                neg     edx
@@5210:         popf
                jnz     @@5170
                lea     edi,NU
                mov     ecx,la1
@@5250:         cmp     dword ptr [edi+ecx*4],0
                jne     @@5260
                dec     ecx
                jnz     @@5250
                inc     ecx
@@5260:         mov     [edi],ecx
                jmp     @@5120
@@5130:         mov     edx,la1
                lea     esi,NA
                lea     ebx,NU
@@5270:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                mov     dword ptr [ebx+ecx*4],0
                dec     ecx
                jnz     @@5270
@@5280:         lea     esi,[esi+edx*4+4]
                mov     ecx,la2
                sub     edx,ecx
                lea     edi,NB
                add     edi,4
                lea     edi,[edi+ecx*4]
                mov     eax,[esi-4]
                cmp     eax,[edi-4]
                jb      @@5290
                mov     dword ptr [esi],0
                add     esi,4
                inc     edx
                jmp     @@5300
@@5290:         lea     ebx,NU
                mov     dword ptr [ebx+edx*4],0
                test    edx,edx
                jz      @@5310
@@5300:         mov     ebx,[edi-8]
                mov     eax,[edi-4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                mov     reg_a,eax
                mov     eax,[edi-12]
                shld    ebx,eax,cl
                mov     reg_b,ebx
                mov     reg_s,ecx
                mov     reg_c,edx
@@5320:         sub     esi,4
                mov     edx,[esi]
                mov     ebx,[esi-4]
                mov     eax,[esi-8]
                mov     ecx,reg_s
                shld    edx,ebx,cl
                shld    ebx,eax,cl
                mov     reg_u,ebx
                mov     ebx,[esi-12]
                shld    eax,ebx,cl
                mov     ebx,eax
                mov     eax,reg_u
                cmp     reg_a,edx
                je      @@5330
                div     reg_a
                mov     reg_q,eax
                jmp     @@5340
@@5330:         add     edx,eax
                mov     reg_q,4294967295
                jc      @@5350
@@5340:         mov     ecx,edx
                mov     eax,reg_b
                mul     reg_q
                sub     ebx,eax
                sbb     ecx,edx
                jnb     @@5350
                dec     reg_q
                add     ebx,reg_b
                adc     ecx,reg_a
                jc      @@5350
                dec     reg_q
@@5350:         mov     ecx,la2
                neg     ecx
                xor     ebx,ebx
@@5360:         mov     eax,[edi+ecx*4]
                mul     reg_q
                add     eax,ebx
                adc     edx,0
                sub     [esi+ecx*4],eax
                adc     edx,0
                mov     ebx,edx
                inc     ecx
                jne     @@5360
                cmp     [esi],ebx
                je      @@5370
                dec     reg_q
                mov     ecx,la2
                neg     ecx
                clc
@@5380:         mov     eax,[edi+ecx*4]
                adc     [esi+ecx*4],eax
                inc     ecx
                jne     @@5380
@@5370:         lea     ebx,NU
                mov     edx,reg_c
                mov     eax,reg_q
                mov     [ebx+edx*4],eax
                dec     reg_c
                jne     @@5320
@@5310:         mov     ecx,la1
@@5390:         cmp     dword ptr[ebx+ecx*4],0
                jne     @@5400
                dec     ecx
                jnz     @@5390
                inc     ecx
@@5400:         mov     [ebx],ecx
@@5120:         lea     esi,NB
                mov     edx,[esi]
                add     esi,4
                lea     edi,NU
                mov     ecx,[edi]
                add     edi,4
                cmp     edx,ecx
                jae     @@5410
                xchg    edi,esi
                xchg    edx,ecx
@@5410:         lea     ebx,NB
                mov     [ebx],edx
                add     ebx,4
                sub     edx,ecx
                jecxz   @@5420
                lea     esi,[esi+ecx*4]
                lea     edi,[edi+ecx*4]
                lea     ebx,[ebx+ecx*4]
                neg     ecx
                clc
@@5430:         mov     eax,[esi+ecx*4]
                adc     eax,[edi+ecx*4]
                mov     [ebx+ecx*4],eax
                inc     ecx
                jne     @@5430
                jnc     @@5420
                test    edx,edx
                jz      @@5440
                lea     esi,[esi+edx*4]
                lea     ebx,[ebx+edx*4]
                neg     edx
@@5450:         mov     eax,[esi+edx*4]
                inc     eax
                jne     @@5460
                mov     [ebx+edx*4],eax
                inc     edx
                jne     @@5450
@@5440:         mov     dword ptr [ebx],1
                lea     ebx,NB
                inc     dword ptr [ebx]
                jmp     @@5470
@@5420:         test    edx,edx
                jz      @@5480
                lea     esi,[esi+edx*4]
                lea     ebx,[ebx+edx*4]
                neg     edx
@@5490:         mov     eax,[esi+edx*4]
@@5460:         mov     [ebx+edx*4],eax
                inc     edx
                jne     @@5490
@@5480:         mov     [ebx],ecx
@@5470:         lea     edi,NB
                mov     ecx,[edi]
                clc
@@5500:         rcr     dword ptr [edi+ecx*4],1
                dec     ecx
                jnz     @@5500
                mov     ecx,[edi]
                cmp     dword ptr [edi+ecx*4],0
                jne     @@5510
                dec     dword ptr [edi]
                jnz     @@5510
                inc     dword ptr [edi]
@@5510:         lea     edi,NB
                lea     esi,NU
                mov     ecx,[edi]
                cmp     ecx,[esi]
                jb      @@5040
                ja      @@5070
@@5520:         mov     eax,[edi+ecx*4]
                cmp     eax,[esi+ecx*4]
                jb      @@5040
                ja      @@5070
                dec     ecx
                jnz     @@5520
@@5040:         lea     esi,NB
                mov     ecx,[esi]
                mov     edi,esi
                mov     edx,ecx
                lea     ebx,NU
                mov     pm,ecx
                add     pm,ecx
                add     esi,4
                add     edi,4
                add     ebx,4
                lea     esi,dword ptr [esi+ecx*4]
                lea     ebx,dword ptr [ebx+ecx*4]
                neg     ecx
                mov     eax,[edi]
                mov     mul0,eax
                mov     loc_lb,edx
                mov     loc_la,ecx
                mov     loc_b,edi
                mov     loc_a,esi
                xor     edi,edi
@@5530:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                mov     [ebx+ecx*4],eax
                mov     edi,edx
                inc     ecx
                jne     @@5530
                mov     [ebx],edx
                dec     loc_lb
                jz      @@5540
@@5550:         add     loc_b,4
                mov     edi,loc_b
                mov     edi,[edi]
                mov     mul0,edi
                mov     esi,loc_a
                add     ebx,4
                mov     ecx,loc_la
                xor     edi,edi
@@5560:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                add     [ebx+ecx*4],eax
                adc     edx,0
                mov     edi,edx
                inc     ecx
                jne     @@5560
                mov     [ebx],edx
                dec     loc_lb
                jnz     @@5550
@@5540:         lea     esi,NU
                mov     ecx,pm
@@5570:         cmp     dword ptr[esi+ecx*4],0
                jne     @@5580
                dec     ecx
                jnz     @@5570
                inc     ecx
@@5580:         mov     [esi],ecx
                mov     edi,[ebp+8]
                lea     esi,NU
                mov     ecx,[edi]
                cmp     ecx,[esi]
                jne     @@5599
@@5590:         mov     ebx,[edi+ecx*4]
                cmp     ebx,[esi+ecx*4]
                jne     @@5599
                dec     ecx
                jnz     @@5590
                jmp     @@100000
@@5599:         mov     esi,reg_esi
@@6000:         mov     eax,tk
                mul     eax
                sub     eax,4
                mov     ebx,eax
                mov     ecx,dimn
                xor     edx,edx
@@6010:         mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@6010
                cmp     edx,0
                je      @@100000
@@6011:         bsf     ecx,edx
                shr     edx,cl
                test    ecx,1
                jz      @@6030
                mov     eax,ebx
                and     eax,7
                cmp     al,3
                je      @@6025
                cmp     al,5
                jne     @@6030
@@6025:         neg     esi
@@6030:         cmp     edx,1
                jne     @@6040
                cmp     esi,1
                je      @@4010
                jmp     @@6100
@@6040:         mov     ecx,edx
                mov     eax,ebx
                xor     edx,edx
                div     ecx
                cmp     edx,0
                je      @@100000
                mov     eax,ebx
                and     eax,3
                cmp     eax,3
                jne     @@6050
                mov     eax,ecx
                and     eax,3
                cmp     eax,3
                jne     @@6050
                neg     esi
@@6050:         mov     ebx,ecx
                jmp     @@6011
@@6100:         mov     ecx,[edi]
                lea     esi,NU
                mov     [esi],ecx
@@6150:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@6150
                mov     ecx,[edi]
@@6160:         add     esi,4
                inc     dword ptr [esi]
                jnz     @@6170
                dec     ecx
                jnz     @@6160
                mov     dword ptr [esi+4],1
                lea     esi,NU
                inc     dword ptr [esi]
@@6170:         lea     esi,NU
                mov     ecx,[esi]
                xor     edx,edx
                lea     esi,[esi+ecx*4+4]
                neg     ecx
                mov     eax,[esi+ecx*4]
                cmp     eax,0
                jne     @@6500
                inc     ecx
                jz      @@6510
                mov     edx,32
@@6520:         mov     eax,[esi+ecx*4]
                cmp     eax,0
                jne     @@6500
                add     edx,32
                inc     ecx
                jnz     @@6520
                xor     edx,edx
                jmp     @@6510
@@6500:         bsf     ecx,eax
                add     edx,ecx
@@6510:         mov     u,edx
                lea     edi,NU
                mov     esi,edi
                mov     ecx,edx
                mov     edx,[edi]
                cmp     ecx,0
                je      @@6530
@@6550:         cmp     ecx,32
                jb      @@6540
                mov     eax,ecx
                shr     eax,5
                cmp     eax,edx
                jb      @@6560
                mov     dword ptr [edi],1
                mov     dword ptr [edi+4],0
                jmp     @@6530
@@6560:         and     ecx,31
                jnz     @@6570
                mov     ecx,edx
                sub     ecx,eax
                mov     [edi],ecx
                lea     esi,NA
@@6580:         mov     eax,[edi+edx*4]
                mov     [esi+ecx*4],eax
                dec     edx
                dec     ecx
                jnz     @@6580
                mov     ecx,[edi]
@@6590:         mov     eax,[esi+ecx*4]
                mov     [edi+ecx*4],eax
                dec     ecx
                jnz     @@6590
                jmp     @@6530
@@6570:         sub     edx,eax
                lea     edi,[edi+eax*4]
@@6540:         mov     [esi],edx
                lea     edi,[edi+edx*4+4]
                lea     esi,[esi+edx*4]
                neg     edx
                mov     ebx,[edi+edx*4]
                inc     edx
                jnz     @@6600
                shr     ebx,cl
                mov     [esi],ebx
                jmp     @@6530
@@6600:         mov     eax,[edi+edx*4]
                shrd    ebx,eax,cl
                mov     [esi+edx*4],ebx
                mov     ebx,eax
                inc     edx
                jnz     @@6600
                shr     ebx,cl
                mov     [esi],ebx
                lea     edi,NU
@@6610:         mov     ecx,[edi]
                cmp     dword ptr [edi+ecx*4],0
                jne     @@6530
                dec     ecx
                jz      @@6530
@@6620:         cmp     dword ptr [edi+ecx*4],0
                jne     @@6630
                dec     ecx
                jnz     @@6620
                inc     ecx
@@6630:         mov     [edi],ecx
@@6530:         mov     edi,[ebp+8]
                lea     esi,NV
                mov     ecx,[edi]
                mov     edx,ecx
                mov     [esi],ecx
@@7001:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@7001
                mov     eax,2
                add     esi,4
                sub     [esi],eax
                jnc     @@7005
                dec     edx
@@7002:         add     esi,4
                sbb     dword ptr[esi],1
                jnc     @@7003
                dec     edx
                jnz     @@7002
                jmp     @@7005
@@7003:         jnz     @@7005
                dec     edx
                jnz     @@7005
                lea     edi,NV
                dec     dword ptr [edi]
@@7005:         lea     esi,NT
                mov     eax,tk
                mov     dword ptr [esi+4],eax
                mul     eax
                sub     eax,2
                lea     edi,NY
                mov     dword ptr [edi],1
                mov     dword ptr [edi+4],eax
                lea     edi,NU
                mov     ecx,[edi]
                bsr     eax,[edi+ecx*4]
                dec     ecx
                shl     ecx,5
                add     eax,ecx
                jz      @@50000
                dec     eax
                jz      @@50000
                mov     h,eax
@@7800:         lea     esi,NU
                mov     ecx,h
                mov     ebx,ecx
                and     ebx,31
                shr     ecx,5
                inc     ecx
                mov     eax,[esi+ecx*4]
                bt      eax,ebx
                jnc     @@19990
                lea     esi,NT
                mov     ecx,[esi]
                lea     edi,NY
                mov     edx,[edi]
                lea     ebx,NA
                cmp     ecx,edx
                jae     @@8000
                xchg    edi,esi
                xchg    edx,ecx
@@8000:         mov     pm,ecx
                add     pm,edx
                add     esi,4
                add     edi,4
                add     ebx,4
                lea     esi,dword ptr [esi+ecx*4]
                lea     ebx,dword ptr [ebx+ecx*4]
                neg     ecx
                mov     eax,[edi]
                mov     mul0,eax
                mov     loc_lb,edx
                mov     loc_la,ecx
                mov     loc_b,edi
                mov     loc_a,esi
                xor     edi,edi
@@8010:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                mov     [ebx+ecx*4],eax
                mov     edi,edx
                inc     ecx
                jne     @@8010
                mov     [ebx],edx
                dec     loc_lb
                jz      @@8020
@@8030:         add     loc_b,4
                mov     edi,loc_b
                mov     edi,[edi]
                mov     mul0,edi
                mov     esi,loc_a
                add     ebx,4
                mov     ecx,loc_la
                xor     edi,edi
@@8040:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                add     [ebx+ecx*4],eax
                adc     edx,0
                mov     edi,edx
                inc     ecx
                jne     @@8040
                mov     [ebx],edx
                dec     loc_lb
                jnz     @@8030
@@8020:         lea     edi,NA
                mov     ecx,pm
@@8050:         cmp     dword ptr[edi+ecx*4],0
                jne     @@8060
                dec     ecx
                jnz     @@8050
                inc     ecx
@@8060:         mov     [edi],ecx
                mov     eax,tk
                cmp     ecx,1
                jne     @@8070
                mov     ebx,[edi+4]
                lea     esi,NT
                sub     eax,ebx
                ja      @@8080
                neg     eax
                mov     dword ptr [esi],1
                mov     dword ptr [esi+4],eax
                jmp     @@8090
@@8080:         mov     edi,[ebp+8]
                mov     ecx,[edi]
                mov     edx,ecx
                mov     [esi],ecx
@@8100:         mov     ebx,[edi+ecx*4]
                mov     [esi+ecx*4],ebx
                dec     ecx
                jnz     @@8100
                add     esi,4
                sub     [esi],eax
                jnc     @@8090
                dec     edx
@@8110:         add     esi,4
                sbb     dword ptr [esi],1
                jnc     @@8120
                dec     edx
                jnz     @@8110
                jmp     @@8090
@@8120:         jnz     @@8090
                dec     edx
                jnz     @@8090
                lea     esi,NT
                dec     dword ptr [esi]
                jmp     @@8090
@@8070:         mov     edx,ecx
                add     edi,4
                sub     [edi],eax
                jnc     @@8130
                dec     edx
@@8140:         add     edi,4
                sbb     dword ptr [edi],1
                jnc     @@8150
                dec     edx
                jnz     @@8140
                jmp     @@8130
@@8150:         jnz     @@8130
                dec     edx
                jnz     @@8130
                lea     edi,NA
                dec     dword ptr [edi]
@@8130:         lea     edi,NA
@@8160:         mov     la1,ecx
                mov     esi,[ebp+8]
                mov     edx,[esi]
                cmp     edx,la1
                jbe     @@8170
                lea     esi,NT
                mov     [esi],ecx
@@8180:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@8180
                jmp     @@8090
@@8170:         mov     la2,edx
                cmp     edx,2
                ja      @@8190
                je      @@8200
                mov     ebx,[esi+4]
                xor     edx,edx
@@8210:         mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@8210
                lea     ebx,NT
                mov     dword ptr [ebx],1
                mov     [ebx+4],edx
                jmp     @@8090
@@8200:         mov     ebx,[esi+4]
                mov     eax,[esi+8]
                add     edi,4
                xor     edx,edx
                dec     ecx
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@8220:         cmp     edx,reg_b
                je      @@8230
                div     reg_b
                jmp     @@8240
@@8230:         add     edx,eax
                mov     eax,4294967295
                jnc     @@8240
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@8250
                mov     ebx,[edi]
@@8250:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@8260
@@8240:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@8270
                mov     ebx,[edi]
@@8270:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@8280
                ja      @@8290
                test    eax,eax
                jz      @@8280
@@8290:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@8280
                ja      @@8290
                test    eax,eax
                jnz     @@8290
@@8280:         neg     eax
                adc     edx,0
                neg     edx
@@8260:         popf
                jnz     @@8220
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,NT
                mov     [edi+4],eax
                cmp     edx,0
                jne     @@8300
                mov     dword ptr [edi],1
                jmp     @@8090
@@8300:         mov     [edi+8],edx
                mov     dword ptr [edi],2
                jmp     @@8090
@@8190:         mov     edx,la1
                lea     esi,NT
                mov     [esi],ecx
@@8310:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@8310
@@8320:         lea     esi,[esi+edx*4+4]
                mov     ecx,la2
                sub     edx,ecx
                mov     edi,[ebp+8]
                lea     edi,[edi+ecx*4+4]
                mov     eax,[esi-4]
                cmp     eax,[edi-4]
                jb      @@8330
                mov     dword ptr [esi],0
                add     esi,4
                inc     edx
                jmp     @@8340
@@8330:         test    edx,edx
                jz      @@8350
@@8340:         mov     ebx,[edi-8]
                mov     eax,[edi-4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                mov     reg_a,eax
                mov     eax,[edi-12]
                shld    ebx,eax,cl
                mov     reg_b,ebx
                mov     reg_s,ecx
                mov     reg_c,edx
@@8360:         sub     esi,4
                mov     edx,[esi]
                mov     ebx,[esi-4]
                mov     eax,[esi-8]
                mov     ecx,reg_s
                shld    edx,ebx,cl
                shld    ebx,eax,cl
                mov     reg_u,ebx
                mov     ebx,[esi-12]
                shld    eax,ebx,cl
                mov     ebx,eax
                mov     eax,reg_u
                cmp     reg_a,edx
                je      @@8370
                div     reg_a
                mov     reg_q,eax
                jmp     @@8380
@@8370:         add     edx,eax
                mov     reg_q,4294967295
                jc      @@8390
@@8380:         mov     ecx,edx
                mov     eax,reg_b
                mul     reg_q
                sub     ebx,eax
                sbb     ecx,edx
                jnb     @@8390
                dec     reg_q
                add     ebx,reg_b
                adc     ecx,reg_a
                jc      @@8390
                dec     reg_q
@@8390:         mov     ecx,la2
                neg     ecx
                xor     ebx,ebx
@@8400:         mov     eax,[edi+ecx*4]
                mul     reg_q
                add     eax,ebx
                adc     edx,0
                sub     [esi+ecx*4],eax
                adc     edx,0
                mov     ebx,edx
                inc     ecx
                jne     @@8400
                cmp     [esi],ebx
                je      @@8410
                dec     reg_q
                mov     ecx,la2
                neg     ecx
                clc
@@8420:         mov     eax,[edi+ecx*4]
                adc     [esi+ecx*4],eax
                inc     ecx
                jne     @@8420
@@8410:         dec     reg_c
                jne     @@8360
@@8350:         lea     esi,NT
                mov     ecx,la2
@@8430:         cmp     dword ptr[esi+ecx*4],0
                jne     @@8440
                dec     ecx
                jnz     @@8430
                inc     ecx
@@8440:         mov     [esi],ecx
@@8090:         lea     edi,NY
                cmp     dword ptr [edi+4],65535
                ja      @@9000
                cmp     dword ptr [edi],1
                jne     @@9000
                mov     eax,[edi+4]
                cmp     eax,2
                jb      @@9010
                mul     eax
                sub     eax,2
                mov     [edi+4],eax
                jmp     @@9020
@@9010:         mov     ebx,2
                sub     ebx,eax
                mov     esi,[ebp+8]
                mov     ecx,[esi]
                mov     edx,ecx
                mov     [edi],ecx
@@9030:         mov     eax,[esi+ecx*4]
                mov     [edi+ecx*4],eax
                dec     ecx
                jnz     @@9030
                add     edi,4
                sub     [edi],ebx
                jnc     @@9020
                dec     edx
@@9040:         add     edi,4
                sbb     dword ptr [edi],1
                jnc     @@9050
                dec     edx
                jnz     @@9040
                jmp     @@9020
@@9050:         jnz     @@9020
                dec     edx
                jnz     @@9020
                lea     edi,NY
                dec     dword ptr [edi]
                jmp     @@9020
@@9000:         mov     esi,edi
                mov     ecx,[edi]
                mov     edx,ecx
                lea     ebx,NA
                mov     pm,ecx
                add     pm,ecx
                add     esi,4
                add     edi,4
                add     ebx,4
                lea     esi,dword ptr [esi+ecx*4]
                lea     ebx,dword ptr [ebx+ecx*4]
                neg     ecx
                mov     eax,[edi]
                mov     mul0,eax
                mov     loc_lb,edx
                mov     loc_la,ecx
                mov     loc_b,edi
                mov     loc_a,esi
                xor     edi,edi
@@9060:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                mov     [ebx+ecx*4],eax
                mov     edi,edx
                inc     ecx
                jne     @@9060
                mov     [ebx],edx
                dec     loc_lb
                jz      @@9070
@@9080:         add     loc_b,4
                mov     edi,loc_b
                mov     edi,[edi]
                mov     mul0,edi
                mov     esi,loc_a
                add     ebx,4
                mov     ecx,loc_la
                xor     edi,edi
@@9090:         mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                add     [ebx+ecx*4],eax
                adc     edx,0
                mov     edi,edx
                inc     ecx
                jne     @@9090
                mov     [ebx],edx
                dec     loc_lb
                jnz     @@9080
@@9070:         lea     edi,NA
                mov     ecx,pm
@@9100:         cmp     dword ptr[edi+ecx*4],0
                jne     @@9110
                dec     ecx
                jnz     @@9100
                inc     ecx
@@9110:         mov     [edi],ecx
                add     edi,4
                sub     dword ptr [edi],2
                jnc     @@9120
                dec     ecx
@@9130:         add     edi,4
                sbb     dword ptr[edi],1
                jnc     @@9140
                dec     ecx
                jnz     @@9130
                jmp     @@9120
@@9140:         jnz     @@9120
                dec     ecx
                jnz     @@9120
                lea     edi,NA
                dec     dword ptr [edi]
@@9120:         lea     edi,NA
                mov     ecx,[edi]
                mov     la1,ecx
                mov     esi,[ebp+8]
                mov     edx,[esi]
                cmp     edx,ecx
                jbe     @@9150
                lea     esi,NY
                mov     [esi],ecx
@@9160:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@9160
                jmp     @@9020
@@9150:         mov     la2,edx
                cmp     edx,2
                ja      @@9170
                je      @@9180
                mov     ebx,[esi+4]
                xor     edx,edx
@@9190:         mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@9190
                lea     ebx,NY
                mov     dword ptr [ebx],1
                mov     [ebx+4],edx
                jmp     @@9020
@@9180:         mov     ebx,[esi+4]
                mov     eax,[esi+8]
                add     edi,4
                xor     edx,edx
                dec     ecx
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@9200:         cmp     edx,reg_b
                je      @@9210
                div     reg_b
                jmp     @@9220
@@9210:         add     edx,eax
                mov     eax,4294967295
                jnc     @@9220
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@9230
                mov     ebx,[edi]
@@9230:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@9240
@@9220:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@9250
                mov     ebx,[edi]
@@9250:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@9260
                ja      @@9270
                test    eax,eax
                jz      @@9260
@@9270:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@9260
                ja      @@9270
                test    eax,eax
                jnz     @@9270
@@9260:         neg     eax
                adc     edx,0
                neg     edx
@@9240:         popf
                jnz     @@9200
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,NY
                mov     [edi+4],eax
                cmp     edx,0
                jne     @@9280
                mov     dword ptr [edi],1
                jmp     @@9020
@@9280:         mov     [edi+8],edx
                mov     dword ptr [edi],2
                jmp     @@9020
@@9170:         mov     edx,la1
                lea     esi,NY
                mov     [esi],ecx
@@9290:         mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@9290
@@9300:         lea     esi,[esi+edx*4+4]
                mov     ecx,la2
                sub     edx,ecx
                mov     edi,[ebp+8]
                lea     edi,[edi+ecx*4+4]
                mov     eax,[esi-4]
                cmp     eax,[edi-4]
                jb      @@9310
                mov     dword ptr [esi],0
                add     esi,4
                inc     edx
                jmp     @@9320
@@9310:         test    edx,edx
                jz      @@9330
@@9320:         mov     ebx,[edi-8]
                mov     eax,[edi-4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                mov     reg_a,eax
                mov     eax,[edi-12]
                shld    ebx,eax,cl
                mov     reg_b,ebx
                mov     reg_s,ecx
                mov     reg_c,edx
@@9340:         sub     esi,4
                mov     edx,[esi]
                mov     ebx,[esi-4]
                mov     eax,[esi-8]
                mov     ecx,reg_s
                shld    edx,ebx,cl
                shld    ebx,eax,cl
                mov     reg_u,ebx
                mov     ebx,[esi-12]
                shld    eax,ebx,cl
                mov     ebx,eax
                mov     eax,reg_u
                cmp     reg_a,edx
                je      @@9350
                div     reg_a
                mov     reg_q,eax
                jmp     @@9360
@@9350:         add     edx,eax
                mov     reg_q,4294967295
                jc      @@9370
@@9360:         mov     ecx,edx
                mov     eax,reg_b
                mul     reg_q
                sub     ebx,eax
                sbb     ecx,edx
                jnb     @@9370
                dec     reg_q
                add     ebx,reg_b
                adc     ecx,reg_a
                jc      @@9370
                dec     reg_q
@@9370:         mov     ecx,la2
                neg     ecx
                xor     ebx,ebx
@@9380:         mov     eax,[edi+ecx*4]
                mul     reg_q
                add     eax,ebx
                adc     edx,0
                sub     [esi+ecx*4],eax
                adc     edx,0
                mov     ebx,edx
                inc     ecx
                jne     @@9380
                cmp     [esi],ebx
                je      @@9390
                dec     reg_q
                mov     ecx,la2
                neg     ecx
                clc
@@9400:         mov     eax,[edi+ecx*4]
                adc     [esi+ecx*4],eax
                inc     ecx
                jne     @@9400
@@9390:         dec     reg_c
                jne     @@9340
@@9330:         lea     esi,NY
                mov     ecx,la2
@@9410:         cmp     dword ptr[esi+ecx*4],0
                jne     @@9420
                dec     ecx
                jnz     @@9410
                inc     ecx
@@9420:         mov     [esi],ecx
@@9020:         dec     h
                jns     @@7800
                jmp     @@50000
@@19990:        lea     esi,NT
                mov     ecx,[esi]
                lea     edi,NY
                mov     edx,[edi]
                lea     ebx,NA
                cmp     ecx,edx
                jae     @@20000
                xchg    edi,esi
                xchg    edx,ecx
@@20000:        mov     pm,ecx
                add     pm,edx
                add     esi,4
                add     edi,4
                add     ebx,4
                lea     esi,dword ptr [esi+ecx*4]
                lea     ebx,dword ptr [ebx+ecx*4]
                neg     ecx
                mov     eax,[edi]
                mov     mul0,eax
                mov     loc_lb,edx
                mov     loc_la,ecx
                mov     loc_b,edi
                mov     loc_a,esi
                xor     edi,edi
@@20010:        mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                mov     [ebx+ecx*4],eax
                mov     edi,edx
                inc     ecx
                jne     @@20010
                mov     [ebx],edx
                dec     loc_lb
                jz      @@20020
@@20030:        add     loc_b,4
                mov     edi,loc_b
                mov     edi,[edi]
                mov     mul0,edi
                mov     esi,loc_a
                add     ebx,4
                mov     ecx,loc_la
                xor     edi,edi
@@20040:        mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                add     [ebx+ecx*4],eax
                adc     edx,0
                mov     edi,edx
                inc     ecx
                jne     @@20040
                mov     [ebx],edx
                dec     loc_lb
                jnz     @@20030
@@20020:        lea     edi,NA
                mov     ecx,pm
@@20050:        cmp     dword ptr[edi+ecx*4],0
                jne     @@20060
                dec     ecx
                jnz     @@20050
                inc     ecx
@@20060:        mov     [edi],ecx
                mov     eax,tk
                cmp     ecx,1
                jne     @@20070
                mov     ebx,[edi+4]
                lea     esi,NY
                sub     eax,ebx
                ja      @@20080
                neg     eax
                mov     dword ptr [esi],1
                mov     dword ptr [esi+4],eax
                jmp     @@20090
@@20080:        mov     edi,[ebp+8]
                mov     ecx,[edi]
                mov     edx,ecx
                mov     [esi],ecx
@@20100:        mov     ebx,[edi+ecx*4]
                mov     [esi+ecx*4],ebx
                dec     ecx
                jnz     @@20100
                add     esi,4
                sub     [esi],eax
                jnc     @@20090
                dec     edx
@@20110:        add     esi,4
                sbb     dword ptr [esi],1
                jnc     @@20120
                dec     edx
                jnz     @@20110
                jmp     @@20090
@@20120:        jnz     @@20090
                dec     edx
                jnz     @@20090
                lea     esi,NY
                dec     dword ptr [esi]
                jmp     @@20090
@@20070:        mov     edx,ecx
                add     edi,4
                sub     [edi],eax
                jnc     @@20130
                dec     edx
@@20140:        add     edi,4
                sbb     dword ptr [edi],1
                jnc     @@20150
                dec     edx
                jnz     @@20140
                jmp     @@20130
@@20150:        jnz     @@20130
                dec     edx
                jnz     @@20130
                lea     edi,NA
                dec     dword ptr [edi]
@@20130:        lea     edi,NA
@@20160:        mov     la1,ecx
                mov     esi,[ebp+8]
                mov     edx,[esi]
                cmp     edx,la1
                jbe     @@20170
                lea     esi,NY
                mov     [esi],ecx
@@20180:        mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@20180
                jmp     @@20090
@@20170:        mov     la2,edx
                cmp     edx,2
                ja      @@20190
                je      @@20200
                mov     ebx,[esi+4]
                xor     edx,edx
@@20210:        mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@20210
                lea     ebx,NY
                mov     dword ptr [ebx],1
                mov     [ebx+4],edx
                jmp     @@20090
@@20200:        mov     ebx,[esi+4]
                mov     eax,[esi+8]
                add     edi,4
                xor     edx,edx
                dec     ecx
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@20220:        cmp     edx,reg_b
                je      @@20230
                div     reg_b
                jmp     @@20240
@@20230:        add     edx,eax
                mov     eax,4294967295
                jnc     @@20240
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@20250
                mov     ebx,[edi]
@@20250:        add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@20260
@@20240:        mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@20270
                mov     ebx,[edi]
@@20270:        shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@20280
                ja      @@20290
                test    eax,eax
                jz      @@20280
@@20290:        dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@20280
                ja      @@20290
                test    eax,eax
                jnz     @@20290
@@20280:        neg     eax
                adc     edx,0
                neg     edx
@@20260:        popf
                jnz     @@20220
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,NY
                mov     [edi+4],eax
                cmp     edx,0
                jne     @@20300
                mov     dword ptr [edi],1
                jmp     @@20090
@@20300:        mov     [edi+8],edx
                mov     dword ptr [edi],2
                jmp     @@20090
@@20190:        mov     edx,la1
                lea     esi,NY
                mov     [esi],ecx
@@20310:        mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@20310
@@20320:        lea     esi,[esi+edx*4+4]
                mov     ecx,la2
                sub     edx,ecx
                mov     edi,[ebp+8]
                lea     edi,[edi+ecx*4+4]
                mov     eax,[esi-4]
                cmp     eax,[edi-4]
                jb      @@20330
                mov     dword ptr [esi],0
                add     esi,4
                inc     edx
                jmp     @@20340
@@20330:        test    edx,edx
                jz      @@20350
@@20340:        mov     ebx,[edi-8]
                mov     eax,[edi-4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                mov     reg_a,eax
                mov     eax,[edi-12]
                shld    ebx,eax,cl
                mov     reg_b,ebx
                mov     reg_s,ecx
                mov     reg_c,edx
@@20360:        sub     esi,4
                mov     edx,[esi]
                mov     ebx,[esi-4]
                mov     eax,[esi-8]
                mov     ecx,reg_s
                shld    edx,ebx,cl
                shld    ebx,eax,cl
                mov     reg_u,ebx
                mov     ebx,[esi-12]
                shld    eax,ebx,cl
                mov     ebx,eax
                mov     eax,reg_u
                cmp     reg_a,edx
                je      @@20370
                div     reg_a
                mov     reg_q,eax
                jmp     @@20380
@@20370:        add     edx,eax
                mov     reg_q,4294967295
                jc      @@20390
@@20380:        mov     ecx,edx
                mov     eax,reg_b
                mul     reg_q
                sub     ebx,eax
                sbb     ecx,edx
                jnb     @@20390
                dec     reg_q
                add     ebx,reg_b
                adc     ecx,reg_a
                jc      @@20390
                dec     reg_q
@@20390:        mov     ecx,la2
                neg     ecx
                xor     ebx,ebx
@@20400:        mov     eax,[edi+ecx*4]
                mul     reg_q
                add     eax,ebx
                adc     edx,0
                sub     [esi+ecx*4],eax
                adc     edx,0
                mov     ebx,edx
                inc     ecx
                jne     @@20400
                cmp     [esi],ebx
                je      @@20410
                dec     reg_q
                mov     ecx,la2
                neg     ecx
                clc
@@20420:        mov     eax,[edi+ecx*4]
                adc     [esi+ecx*4],eax
                inc     ecx
                jne     @@20420
@@20410:        dec     reg_c
                jne     @@20360
@@20350:        lea     esi,NY
                mov     ecx,la2
@@20430:        cmp     dword ptr[esi+ecx*4],0
                jne     @@20440
                dec     ecx
                jnz     @@20430
                inc     ecx
@@20440:        mov     [esi],ecx
@@20090:        lea     edi,NT
                cmp     dword ptr [edi+4],65535
                ja      @@21000
                cmp     dword ptr [edi],1
                jne     @@21000
                mov     eax,[edi+4]
                cmp     eax,2
                jb      @@21010
                mul     eax
                sub     eax,2
                mov     [edi+4],eax
                jmp     @@21020
@@21010:        mov     ebx,2
                sub     ebx,eax
                mov     esi,[ebp+8]
                mov     ecx,[esi]
                mov     edx,ecx
                mov     [edi],ecx
@@21030:        mov     eax,[esi+ecx*4]
                mov     [edi+ecx*4],eax
                dec     ecx
                jnz     @@21030
                add     edi,4
                sub     [edi],ebx
                jnc     @@21020
                dec     edx
@@21040:        add     edi,4
                sbb     dword ptr [edi],1
                jnc     @@21050
                dec     edx
                jnz     @@21040
                jmp     @@21020
@@21050:        jnz     @@21020
                dec     edx
                jnz     @@21020
                lea     edi,NT
                dec     dword ptr [edi]
                jmp     @@21020
@@21000:        mov     esi,edi
                mov     ecx,[edi]
                mov     edx,ecx
                lea     ebx,NA
                mov     pm,ecx
                add     pm,ecx
                add     esi,4
                add     edi,4
                add     ebx,4
                lea     esi,dword ptr [esi+ecx*4]
                lea     ebx,dword ptr [ebx+ecx*4]
                neg     ecx
                mov     eax,[edi]
                mov     mul0,eax
                mov     loc_lb,edx
                mov     loc_la,ecx
                mov     loc_b,edi
                mov     loc_a,esi
                xor     edi,edi
@@21060:        mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                mov     [ebx+ecx*4],eax
                mov     edi,edx
                inc     ecx
                jne     @@21060
                mov     [ebx],edx
                dec     loc_lb
                jz      @@21070
@@21080:        add     loc_b,4
                mov     edi,loc_b
                mov     edi,[edi]
                mov     mul0,edi
                mov     esi,loc_a
                add     ebx,4
                mov     ecx,loc_la
                xor     edi,edi
@@21090:        mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                add     [ebx+ecx*4],eax
                adc     edx,0
                mov     edi,edx
                inc     ecx
                jne     @@21090
                mov     [ebx],edx
                dec     loc_lb
                jnz     @@21080
@@21070:        lea     edi,NA
                mov     ecx,pm
@@21100:        cmp     dword ptr[edi+ecx*4],0
                jne     @@21110
                dec     ecx
                jnz     @@21100
                inc     ecx
@@21110:        mov     [edi],ecx
                add     edi,4
                sub     dword ptr [edi],2
                jnc     @@21120
                dec     ecx
@@21130:        add     edi,4
                sbb     dword ptr[edi],1
                jnc     @@21140
                dec     ecx
                jnz     @@21130
                jmp     @@21120
@@21140:        jnz     @@21120
                dec     ecx
                jnz     @@21120
                lea     edi,NA
                dec     dword ptr [edi]
@@21120:        lea     edi,NA
                mov     ecx,[edi]
                mov     la1,ecx
                mov     esi,[ebp+8]
                mov     edx,[esi]
                cmp     edx,ecx
                jbe     @@21150
                lea     esi,NT
                mov     [esi],ecx
@@21160:        mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@21160
                jmp     @@21020
@@21150:        mov     la2,edx
                cmp     edx,2
                ja      @@21170
                je      @@21180
                mov     ebx,[esi+4]
                xor     edx,edx
@@21190:        mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@21190
                lea     ebx,NT
                mov     dword ptr [ebx],1
                mov     [ebx+4],edx
                jmp     @@21020
@@21180:        mov     ebx,[esi+4]
                mov     eax,[esi+8]
                add     edi,4
                xor     edx,edx
                dec     ecx
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@21200:        cmp     edx,reg_b
                je      @@21210
                div     reg_b
                jmp     @@21220
@@21210:        add     edx,eax
                mov     eax,4294967295
                jnc     @@21220
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@21230
                mov     ebx,[edi]
@@21230:        add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@21240
@@21220:        mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@21250
                mov     ebx,[edi]
@@21250:        shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@21260
                ja      @@21270
                test    eax,eax
                jz      @@21260
@@21270:        dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@21260
                ja      @@21270
                test    eax,eax
                jnz     @@21270
@@21260:        neg     eax
                adc     edx,0
                neg     edx
@@21240:        popf
                jnz     @@21200
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,NT
                mov     [edi+4],eax
                cmp     edx,0
                jne     @@21280
                mov     dword ptr [edi],1
                jmp     @@21020
@@21280:        mov     [edi+8],edx
                mov     dword ptr [edi],2
                jmp     @@21020
@@21170:        mov     edx,la1
                lea     esi,NT
                mov     [esi],ecx
@@21290:        mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@21290
@@21300:        lea     esi,[esi+edx*4+4]
                mov     ecx,la2
                sub     edx,ecx
                mov     edi,[ebp+8]
                lea     edi,[edi+ecx*4+4]
                mov     eax,[esi-4]
                cmp     eax,[edi-4]
                jb      @@21310
                mov     dword ptr [esi],0
                add     esi,4
                inc     edx
                jmp     @@21320
@@21310:        test    edx,edx
                jz      @@21330
@@21320:        mov     ebx,[edi-8]
                mov     eax,[edi-4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                mov     reg_a,eax
                mov     eax,[edi-12]
                shld    ebx,eax,cl
                mov     reg_b,ebx
                mov     reg_s,ecx
                mov     reg_c,edx
@@21340:        sub     esi,4
                mov     edx,[esi]
                mov     ebx,[esi-4]
                mov     eax,[esi-8]
                mov     ecx,reg_s
                shld    edx,ebx,cl
                shld    ebx,eax,cl
                mov     reg_u,ebx
                mov     ebx,[esi-12]
                shld    eax,ebx,cl
                mov     ebx,eax
                mov     eax,reg_u
                cmp     reg_a,edx
                je      @@21350
                div     reg_a
                mov     reg_q,eax
                jmp     @@21360
@@21350:        add     edx,eax
                mov     reg_q,4294967295
                jc      @@21370
@@21360:        mov     ecx,edx
                mov     eax,reg_b
                mul     reg_q
                sub     ebx,eax
                sbb     ecx,edx
                jnb     @@21370
                dec     reg_q
                add     ebx,reg_b
                adc     ecx,reg_a
                jc      @@21370
                dec     reg_q
@@21370:        mov     ecx,la2
                neg     ecx
                xor     ebx,ebx
@@21380:        mov     eax,[edi+ecx*4]
                mul     reg_q
                add     eax,ebx
                adc     edx,0
                sub     [esi+ecx*4],eax
                adc     edx,0
                mov     ebx,edx
                inc     ecx
                jne     @@21380
                cmp     [esi],ebx
                je      @@21390
                dec     reg_q
                mov     ecx,la2
                neg     ecx
                clc
@@21400:        mov     eax,[edi+ecx*4]
                adc     [esi+ecx*4],eax
                inc     ecx
                jne     @@21400
@@21390:        dec     reg_c
                jne     @@21340
@@21330:        lea     esi,NT
                mov     ecx,la2
@@21410:        cmp     dword ptr[esi+ecx*4],0
                jne     @@21420
                dec     ecx
                jnz     @@21410
                inc     ecx
@@21420:        mov     [esi],ecx
@@21020:        dec     h
                jns     @@7800
@@50000:        lea     esi,NT
                cmp     dword ptr [esi],1
                jne     @@50100
                cmp     dword ptr [esi+4],2
                jne     @@50100
                mov     eax,1
                jmp     @@100010
@@50100:        lea     edi,NT
                lea     esi,NV
                mov     ecx,[edi]
                cmp     ecx,[esi]
                jne     @@50120
@@50110:        mov     eax,[edi+ecx*4]
                cmp     eax,[esi+ecx*4]
                jne     @@50120
                dec     ecx
                jnz     @@50110
                mov     eax,1
                jmp     @@100010
@@50120:        dec     u
                jz      @@100000
@@50121:        lea     edi,NT
                cmp     dword ptr [edi+4],65535
                ja      @@51000
                cmp     dword ptr [edi],1
                jne     @@51000
                mov     eax,[edi+4]
                cmp     eax,2
                jb      @@51010
                mul     eax
                sub     eax,2
                mov     [edi+4],eax
                jmp     @@51020
@@51010:        mov     ebx,2
                sub     ebx,eax
                mov     esi,[ebp+8]
                mov     ecx,[esi]
                mov     edx,ecx
                mov     [edi],ecx
@@51030:        mov     eax,[esi+ecx*4]
                mov     [edi+ecx*4],eax
                dec     ecx
                jnz     @@51030
                add     edi,4
                sub     [edi],ebx
                jnc     @@51020
                dec     edx
@@51040:        add     edi,4
                sbb     dword ptr [edi],1
                jnc     @@51050
                dec     edx
                jnz     @@51040
                jmp     @@51020
@@51050:        jnz     @@51020
                dec     edx
                jnz     @@51020
                lea     edi,NT
                dec     dword ptr [edi]
                jmp     @@51020
@@51000:        mov     esi,edi
                mov     ecx,[edi]
                mov     edx,ecx
                lea     ebx,NA
                mov     pm,ecx
                add     pm,ecx
                add     esi,4
                add     edi,4
                add     ebx,4
                lea     esi,dword ptr [esi+ecx*4]
                lea     ebx,dword ptr [ebx+ecx*4]
                neg     ecx
                mov     eax,[edi]
                mov     mul0,eax
                mov     loc_lb,edx
                mov     loc_la,ecx
                mov     loc_b,edi
                mov     loc_a,esi
                xor     edi,edi
@@51060:        mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                mov     [ebx+ecx*4],eax
                mov     edi,edx
                inc     ecx
                jne     @@51060
                mov     [ebx],edx
                dec     loc_lb
                jz      @@51070
@@51080:        add     loc_b,4
                mov     edi,loc_b
                mov     edi,[edi]
                mov     mul0,edi
                mov     esi,loc_a
                add     ebx,4
                mov     ecx,loc_la
                xor     edi,edi
@@51090:        mov     eax,mul0
                mul     dword ptr [esi+ecx*4]
                add     eax,edi
                adc     edx,0
                add     [ebx+ecx*4],eax
                adc     edx,0
                mov     edi,edx
                inc     ecx
                jne     @@51090
                mov     [ebx],edx
                dec     loc_lb
                jnz     @@51080
@@51070:        lea     edi,NA
                mov     ecx,pm
@@51100:        cmp     dword ptr[edi+ecx*4],0
                jne     @@51110
                dec     ecx
                jnz     @@51100
                inc     ecx
@@51110:        mov     [edi],ecx
                add     edi,4
                sub     dword ptr [edi],2
                jnc     @@51120
                dec     ecx
@@51130:        add     edi,4
                sbb     dword ptr[edi],1
                jnc     @@51140
                dec     ecx
                jnz     @@51130
                jmp     @@51120
@@51140:        jnz     @@51120
                dec     ecx
                jnz     @@51120
                lea     edi,NA
                dec     dword ptr [edi]
@@51120:        lea     edi,NA
                mov     ecx,[edi]
                mov     la1,ecx
                mov     esi,[ebp+8]
                mov     edx,[esi]
                cmp     edx,ecx
                jbe     @@51150
                lea     esi,NT
                mov     [esi],ecx
@@51160:        mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@51160
                jmp     @@51020
@@51150:        mov     la2,edx
                cmp     edx,2
                ja      @@51170
                je      @@51180
                mov     ebx,[esi+4]
                xor     edx,edx
@@51190:        mov     eax,[edi+ecx*4]
                div     ebx
                dec     ecx
                jnz     @@51190
                lea     ebx,NT
                mov     dword ptr [ebx],1
                mov     [ebx+4],edx
                jmp     @@51020
@@51180:        mov     ebx,[esi+4]
                mov     eax,[esi+8]
                add     edi,4
                xor     edx,edx
                dec     ecx
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@51200:        cmp     edx,reg_b
                je      @@51210
                div     reg_b
                jmp     @@51220
@@51210:        add     edx,eax
                mov     eax,4294967295
                jnc     @@51220
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@51230
                mov     ebx,[edi]
@@51230:        add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@51240
@@51220:        mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@51250
                mov     ebx,[edi]
@@51250:        shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@51260
                ja      @@51270
                test    eax,eax
                jz      @@51260
@@51270:        dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@51260
                ja      @@51270
                test    eax,eax
                jnz     @@51270
@@51260:        neg     eax
                adc     edx,0
                neg     edx
@@51240:        popf
                jnz     @@51200
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,NT
                mov     [edi+4],eax
                cmp     edx,0
                jne     @@51280
                mov     dword ptr [edi],1
                jmp     @@51020
@@51280:        mov     [edi+8],edx
                mov     dword ptr [edi],2
                jmp     @@51020
@@51170:        mov     edx,la1
                lea     esi,NT
                mov     [esi],ecx
@@51290:        mov     eax,[edi+ecx*4]
                mov     [esi+ecx*4],eax
                dec     ecx
                jnz     @@51290
@@51300:        lea     esi,[esi+edx*4+4]
                mov     ecx,la2
                sub     edx,ecx
                mov     edi,[ebp+8]
                lea     edi,[edi+ecx*4+4]
                mov     eax,[esi-4]
                cmp     eax,[edi-4]
                jb      @@51310
                mov     dword ptr [esi],0
                add     esi,4
                inc     edx
                jmp     @@51320
@@51310:        test    edx,edx
                jz      @@51330
@@51320:        mov     ebx,[edi-8]
                mov     eax,[edi-4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                mov     reg_a,eax
                mov     eax,[edi-12]
                shld    ebx,eax,cl
                mov     reg_b,ebx
                mov     reg_s,ecx
                mov     reg_c,edx
@@51340:        sub     esi,4
                mov     edx,[esi]
                mov     ebx,[esi-4]
                mov     eax,[esi-8]
                mov     ecx,reg_s
                shld    edx,ebx,cl
                shld    ebx,eax,cl
                mov     reg_u,ebx
                mov     ebx,[esi-12]
                shld    eax,ebx,cl
                mov     ebx,eax
                mov     eax,reg_u
                cmp     reg_a,edx
                je      @@51350
                div     reg_a
                mov     reg_q,eax
                jmp     @@51360
@@51350:        add     edx,eax
                mov     reg_q,4294967295
                jc      @@51370
@@51360:        mov     ecx,edx
                mov     eax,reg_b
                mul     reg_q
                sub     ebx,eax
                sbb     ecx,edx
                jnb     @@51370
                dec     reg_q
                add     ebx,reg_b
                adc     ecx,reg_a
                jc      @@51370
                dec     reg_q
@@51370:        mov     ecx,la2
                neg     ecx
                xor     ebx,ebx
@@51380:        mov     eax,[edi+ecx*4]
                mul     reg_q
                add     eax,ebx
                adc     edx,0
                sub     [esi+ecx*4],eax
                adc     edx,0
                mov     ebx,edx
                inc     ecx
                jne     @@51380
                cmp     [esi],ebx
                je      @@51390
                dec     reg_q
                mov     ecx,la2
                neg     ecx
                clc
@@51400:        mov     eax,[edi+ecx*4]
                adc     [esi+ecx*4],eax
                inc     ecx
                jne     @@51400
@@51390:        dec     reg_c
                jne     @@51340
@@51330:        lea     esi,NT
                mov     ecx,la2
@@51410:        cmp     dword ptr[esi+ecx*4],0
                jne     @@51420
                dec     ecx
                jnz     @@51410
                inc     ecx
@@51420:        mov     [esi],ecx
@@51020:        lea     edi,NT
                lea     esi,NV
                mov     ecx,[edi]
                cmp     ecx,[esi]
                jne     @@52120
@@52110:        mov     eax,[edi+ecx*4]
                cmp     eax,[esi+ecx*4]
                jne     @@52120
                dec     ecx
                jnz     @@52110
                mov     eax,1
                jmp     @@100010
@@52120:        cmp     dword ptr [edi],1
                jne     @@52200
                cmp     dword ptr [edi+4],2
                je      @@100000
@@52200:        dec     u
                jnz     @@50121
@@100000:       xor     eax,eax
@@100010:
END;

PROCEDURE next_prime (N: nat; var X: nat);
BEGIN
if (N[0]=1) and (N[1]<30) then begin
X[0]:=1; X[1]:=nxtb[N[1]];
exit
end;
ninc1(N,X);
if N[1] and 1=1 then nincs(X);
cu:=nmod0(X,210);
ct:=cu shr 1;
if Indm[ct]=64 then begin
cv:=cu;
repeat
inc(cv,2);
inc(ct);
until Indm[ct]<>64;
ninc(X,cv-cu);
end;
if is_prime(X)=1 then exit;
cv:=indm[ct];
ninc(X,difm[cv]);
if is_prime(X)=1 then exit;
repeat
if cv=47 then begin
cv:=0;
ninc(X,10)
end
else begin
inc(cv);
ninc(X,difm[cv])
end;
until is_prime(X)=1;
END;

PROCEDURE prec_prime (N: nat; var X: nat);
BEGIN
if (N[0]=1) and (N[1]<30) then begin
X[0]:=1; X[1]:=pxtb[N[1]];
exit
end;
ndec1(N,X);
if N[1] and 1=1 then ndecs(X);
cu:=nmod0(X,210);
ct:=cu shr 1;
if Indm[ct]=64 then begin
cv:=cu;
repeat
dec(cv,2);
dec(ct);
until Indm[ct]<>64;
ndec(X,cu-cv);
end;
if is_prime(X)=1 then exit;
cv:=indm[ct];
repeat
if cv=0 then begin
cv:=47;
ndec(X,2)
end
else begin
dec(cv);
ndec(X,difm[cv])
end;
until is_prime(X)=1
END;

END.
