{Unit to test if a qword is prime
Copyright (C) 2007-2010 Franco Milani
e-mail: nadir@solaris.polarhome.com or astlab@inwind.it
This unit is part of the bigint package v. 1.8 (bigint.zip) available at:
http://www.polarhome.com/~franco/
http://www.polarhome.com:723/~nadir/software/
http://spazioinwind.libero.it/frm/software/
-----------------------------------------------------------------------------
This software is released in the hope that it will be useful, but on
an as-is basis. No warranty of any kind is offered, either expressed
or implied, including, without limitation, the implied warranties of
merchantability and fitness for a particular purpose.
You can use or modify this unit for personal purposes. You can also
redistribute it, but only in the original unmodified form and keeping
intact any part, included the above copyright notice. Possible changes
in the distributed versions will be made only by the author according
to hints, requests or bug reports.
You may distribute software items that link this unit or part of it
provided that you inform the recipients that this unit is employed.
-----------------------------------------------------------------------------
qwprime(n) returns 1 if a qword n is prime or 0 otherwise.
If n<2^32 it uses a Miller-Rabin test to bases 2, 7 and 61. If n>=2^32
the primality is checked by testing if n is a strong pseudoprime to the
base 2 and a Lucas pseudoprime (a Baillie-PSW test, fast and reliable).

References

[1] G.Jaeschke, "On strong pseudoprimes to several bases", Mathematics of
computation, 61 (1993) 915-926
[2] R. Baillie and Wagstaff, Jr, S. S., "Lucas Pseudoprimes", Mathematics
of computation, 35 (1980) 1391-1417
[3] C. Pomerance, J. L. Selfridge and Wagstaff, Jr, S. S., "The pseudoprimes
to 25*10^9", Mathematics of computation, 35 (1980) 1003-1026
-----------------------------------------------------------------------------}
{$packrecords 1}
{$asmmode intel}
{$ifndef ver1_0}{$calling oldfpccall}{$endif}
UNIT isprq;

INTERFACE

FUNCTION qwprime (n: qword): longint;

IMPLEMENTATION

CONST

pk: array [0..168] of longword =
(0,997,991,983,977,971,967,953,947,941,937,929,919,911,907,887,883,881,877,
863,859,857,853,839,829,827,823,821,811,809,797,787,773,769,761,757,751,743,
739,733,727,719,709,701,691,683,677,673,661,659,653,647,643,641,631,619,617,
613,607,601,599,593,587,577,571,569,563,557,547,541,523,521,509,503,499,491,
487,479,467,463,461,457,449,443,439,433,431,421,419,409,401,397,389,383,379,
373,367,359,353,349,347,337,331,317,313,311,307,293,283,281,277,271,269,263,
257,251,241,239,233,229,227,223,211,199,197,193,191,181,179,173,167,163,157,
151,149,139,137,131,127,113,109,107,103,101,97,89,83,79,73,71,67,61,59,53,47,
43,41,37,31,29,23,19,17,13,11,7,5,3,2);

pin: array [0..61] of byte =
(0,0,1,1,0,1,0,1,0,0,0,1,0,1,0,0,0,1,0,1,0,0,0,1,0,0,0,0,0,1,0,1,0,0,0,0,0,1,
0,0,0,1,0,1,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0,1,0,1);

pmq: array [1..16] of byte =
(61,59,53,47,43,41,37,31,29,23,17,13,11,7,5,3);

VAR

nu,nv,nt,ny,nz: qword;
i,u,h,tk,jk,ik,kc,rc,t: longword;
reg_a,reg_b,reg_c,reg_s,reg_u,reg_x,reg_y: longword;
m: array [1..4] of longword;

FUNCTION qwprime (n: qword): longint; assembler;
ASM
                mov     ebx,[ebp+12]
                cmp     ebx,0
                jne     @@1000
                mov     ebx,[ebp+8]
                cmp     ebx,61
                ja      @@1010
                lea     edi,pin
                xor     eax,eax
                mov     al,[edi+ebx]
                jmp     @@1020
@@1010:         test    ebx,1
                jz      @@1030
                mov     h,ebx
                lea     edi,pmq
                mov     esi,15
                xor     ecx,ecx
@@1040:         mov     eax,ebx
                xor     edx,edx
                mov     cl,[edi+esi]
                div     ecx
                cmp     edx,0
                je      @@1030
                dec     esi
                jns     @@1040
                dec     ebx
                mov     jk,ebx
                bsf     ecx,ebx
                mov     u,ecx
                jz      @@1050
                dec     u
@@1050:         shr     ebx,cl
                mov     ik,ebx
                mov     kc,2
                mov     eax,2
                jmp     @@1060
@@1070:         mov     eax,7
                jmp     @@1080
@@1090:         mov     eax,61
@@1080:         mov     ebx,ik
@@1060:         mov     ecx,h
                cmp     ebx,1
                jne     @@1100
                xor     edx,edx
                div     ecx
                mov     eax,edx
                jmp     @@1110
@@1100:         mov     edi,eax
                bsr     edx,ebx
                mov     t,edx
                bt      ebx,0
                jnc     @@1120
                mov     esi,eax
                jmp     @@1130
@@1120:         mov     esi,1
@@1130:         mov     i,1
@@1140:         mov     eax,edi
                mul     edi
                cmp     edx,ecx
                jb      @@1150
                mov     ebx,eax
                mov     eax,edx
                xor     edx,edx
                div     ecx
                mov     eax,ebx
@@1150:         div     ecx
                mov     edi,edx
                mov     eax,ik
                mov     ebx,i
                bt      eax,ebx
                jnc     @@1160
                mov     eax,esi
                mul     edi
                cmp     edx,ecx
                jb      @@1170
                mov     ebx,eax
                mov     eax,edx
                xor     edx,edx
                div     ecx
                mov     eax,ebx
@@1170:         div     ecx
                mov     esi,edx
@@1160:         inc     i
                dec     t
                jnz     @@1140
                mov     eax,esi
@@1110:         cmp     eax,1
                je      @@1180
                cmp     eax,jk
                je      @@1180
                cmp     u,0
                je      @@1030
                mov     esi,u
@@1190:         mul     eax
                cmp     edx,ecx
                jb      @@1200
                mov     ebx,eax
                mov     eax,edx
                xor     edx,edx
                div     ecx
                mov     eax,ebx
@@1200:         div     ecx
                mov     eax,edx
                cmp     eax,jk
                je      @@1180
                cmp     eax,1
                je      @@1030
@@1210:         dec     esi
                jnz     @@1190
                jmp     @@1030
@@1180:         dec     kc
                jz      @@1090
                jns     @@1070
@@1220:         mov     eax,1
                jmp     @@1020
@@1000:         mov     eax,[ebp+8]
                mov     reg_a,eax
                lea     esi,pk
                mov     ecx,168
@@1230:         mov     edi,[esi+ecx*4]
                mov     edx,ebx
                cmp     edx,edi
                jb      @@1240
                mov     eax,edx
                xor     edx,edx
                div     edi
@@1240:         mov     eax,reg_a
                div     edi
                cmp     edx,0
                je      @@1030
                dec     ecx
                jnz     @@1230
                mov     eax,reg_a
                sub     eax,1
                sbb     ebx,0
                lea     esi,nu
                mov     [esi],eax
                mov     [esi+4],ebx
                cmp     eax,0
                je      @@1250
                bsf     ecx,eax
                jmp     @@1260
@@1250:         bsf     ecx,ebx
                add     ecx,32
@@1260:         mov     u,ecx
                cmp     ecx,32
                jb      @@1270
                shrd    eax,ebx,31
                shr     ebx,31
                sub     ecx,31
@@1270:         shrd    eax,ebx,cl
                shr     ebx,cl
                lea     esi,ny
                mov     [esi],eax
                mov     [esi+4],ebx
                mov     eax,2
                mov     ebx,0
                lea     esi,nz
                mov     [esi],eax
                mov     [esi+4],ebx
                lea     esi,nv
                mov     dword ptr [esi],1
                mov     dword ptr [esi+4],0
                mov     ebx,[ebp+12]
                bsr     eax,ebx
                add     eax,32
@@1280:         inc     eax
                neg     eax
                mov     jk,eax
                mov     ik,0
@@1290:         mov     ebx,ik
                lea     edi,ny
                cmp     ebx,32
                jb      @@1300
                mov     eax,[edi+4]
                jmp     @@1310
@@1300:         mov     eax,[edi]
@@1310:         bt      eax,ebx
                jnc     @@1320
                lea     ebx,nv
                lea     esi,nz
                lea     edi,m
                mov     eax,[ebx]
                mov     edx,[esi]
                mul     edx
                mov     ecx,edx
                mov     [edi],eax
                cmp     dword ptr [ebx+4],0
                jne     @@1330
                mov     eax,[ebx]
                mov     edx,[esi+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     [edi+4],ecx
                mov     [edi+8],edx
                mov     dword ptr [edi+12],0
                cmp     edx,0
                je      @@1340
                mov     ecx,2
                jmp     @@1350
@@1340:         cmp     ecx,0
                je      @@1360
                mov     ecx,1
                jmp     @@1350
@@1330:         cmp     dword ptr [esi+4],0
                jne     @@1370
                mov     eax,[esi]
                mov     edx,[ebx+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     [edi+4],ecx
                mov     [edi+8],edx
                mov     dword ptr [edi+12],0
                cmp     edx,0
                je      @@1380
                mov     ecx,2
                jmp     @@1350
@@1380:         cmp     ecx,0
                je      @@1360
                mov     ecx,1
                jmp     @@1350
@@1370:         mov     eax,[esi]
                mov     edx,[ebx+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     h,edx
                mov     eax,[ebx]
                mov     edx,[esi+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     [edi+4],ecx
                mov     ecx,edx
                mov     eax,[ebx+4]
                mov     edx,[esi+4]
                mul     edx
                add     eax,h
                adc     edx,0
                add     eax,ecx
                adc     edx,0
                mov     [edi+8],eax
                mov     [edi+12],edx
                mov     ecx,2
                cmp     edx,0
                je      @@1350
                inc     ecx
                jmp     @@1350
@@1360:         lea     esi,nv
                mov     eax,[edi]
                mov     [esi],eax
                mov     dword ptr [esi+4],0
                jmp     @@1320
@@1350:         mov     reg_c,ecx
                mov     ebx,[ebp+8]
                mov     eax,[ebp+12]
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@1390:         cmp     edx,reg_b
                je      @@1400
                div     reg_b
                jmp     @@1410
@@1400:         add     edx,eax
                mov     eax,4294967295
                jnc     @@1410
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@1420
                mov     ebx,[edi]
@@1420:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@1430
@@1410:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@1440
                mov     ebx,[edi]
@@1440:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@1450
                ja      @@1460
                test    eax,eax
                jz      @@1450
@@1460:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@1450
                ja      @@1460
                test    eax,eax
                jnz     @@1460
@@1450:         neg     eax
                adc     edx,0
                neg     edx
@@1430:         popf
                jnz     @@1390
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,nv
                mov     [edi],eax
                mov     [edi+4],edx
@@1320:         inc     jk
                jz      @@1470
                lea     edi,m
                lea     ebx,nz
                mov     eax,[ebx]
                mul     eax
                mov     [edi],eax
                cmp     dword ptr [ebx+4],0
                jne     @@1480
                mov     [edi+4],edx
                mov     dword ptr [edi+8],0
                mov     dword ptr [edi+12],0
                mov     ecx,0
                cmp     edx,0
                je      @@1490
                inc     ecx
                jmp     @@1500
@@1480:         mov     ecx,edx
                mov     eax,[ebx]
                mov     edx,[ebx+4]
                mul     edx
                mov     reg_a,eax
                mov     reg_b,edx
                add     eax,ecx
                adc     edx,0
                mov     ecx,eax
                mov     esi,edx
                add     ecx,reg_a
                adc     reg_b,0
                mov     [edi+4],ecx
                mov     ecx,reg_b
                mov     eax,[ebx+4]
                mul     eax
                add     eax,ecx
                adc     edx,0
                add     eax,esi
                adc     edx,0
                mov     [edi+8],eax
                mov     [edi+12],edx
                mov     ecx,2
                cmp     edx,0
                je      @@1500
                inc     ecx
                jmp     @@1500
@@1490:         lea     esi,nz
                mov     eax,[edi]
                mov     [esi],eax
                mov     dword ptr [esi+4],0
                inc     ik
                jmp     @@1290
@@1500:         mov     reg_c,ecx
                mov     ebx,[ebp+8]
                mov     eax,[ebp+12]
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@1510:         cmp     edx,reg_b
                je      @@1520
                div     reg_b
                jmp     @@1530
@@1520:         add     edx,eax
                mov     eax,4294967295
                jnc     @@1530
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@1540
                mov     ebx,[edi]
@@1540:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@1550
@@1530:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@1560
                mov     ebx,[edi]
@@1560:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@1570
                ja      @@1580
                test    eax,eax
                jz      @@1570
@@1580:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@1570
                ja      @@1580
                test    eax,eax
                jnz     @@1580
@@1570:         neg     eax
                adc     edx,0
                neg     edx
@@1550:         popf
                jnz     @@1510
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,nz
                mov     [edi],eax
                mov     [edi+4],edx
                inc     ik
                jmp     @@1290
@@1470:         lea     ebx,nv
                mov     eax,[ebx]
                mov     edx,[ebx+4]
                cmp     eax,1
                jne     @@1590
                cmp     edx,0
                je      @@1600
@@1590:         lea     esi,nu
                cmp     eax,[esi]
                jne     @@1610
                cmp     edx,[esi+4]
                je      @@1600
@@1610:         dec     u
                jz      @@1030
                neg     u
@@1620:         mov     reg_x,eax
                mov     reg_y,edx
                lea     edi,m
                mul     eax
                mov     [edi],eax
                cmp     reg_y,0
                jne     @@1630
                mov     [edi+4],edx
                mov     dword ptr [edi+8],0
                mov     dword ptr [edi+12],0
                mov     ecx,0
                cmp     edx,0
                je      @@1640
                inc     ecx
                jmp     @@1650
@@1630:         mov     ecx,edx
                mov     eax,reg_x
                mov     edx,reg_y
                mul     edx
                mov     reg_a,eax
                mov     reg_b,edx
                add     eax,ecx
                adc     edx,0
                mov     ecx,eax
                mov     esi,edx
                add     ecx,reg_a
                adc     reg_b,0
                mov     [edi+4],ecx
                mov     ecx,reg_b
                mov     eax,reg_y
                mul     eax
                add     eax,ecx
                adc     edx,0
                add     eax,esi
                adc     edx,0
                mov     [edi+8],eax
                mov     [edi+12],edx
                mov     ecx,2
                cmp     edx,0
                je      @@1650
                inc     ecx
                jmp     @@1650
@@1640:         mov     eax,[edi]
                mov     edx,0
                jmp     @@1660
@@1650:         mov     ebx,[ebp+8]
                mov     eax,[ebp+12]
                mov     reg_c,ecx
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@1670:         cmp     edx,reg_b
                je      @@1680
                div     reg_b
                jmp     @@1690
@@1680:         add     edx,eax
                mov     eax,4294967295
                jnc     @@1690
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@1700
                mov     ebx,[edi]
@@1700:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@1710
@@1690:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@1720
                mov     ebx,[edi]
@@1720:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@1730
                ja      @@1740
                test    eax,eax
                jz      @@1730
@@1740:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@1730
                ja      @@1740
                test    eax,eax
                jnz     @@1740
@@1730:         neg     eax
                adc     edx,0
                neg     edx
@@1710:         popf
                jnz     @@1670
                shrd    eax,edx,cl
                shr     edx,cl
@@1660:         lea     edi,nu
                cmp     eax,[edi]
                jne     @@1750
                cmp     edx,[edi+4]
                je      @@1600
@@1750:         cmp     eax,1
                jne     @@1760
                cmp     edx,0
                je      @@1030
@@1760:         inc     u
                jnz     @@1620
                jmp     @@1030
@@1600:         mov     edi,1
                mov     esi,edi
@@1770:         inc     edi
                inc     edi
                mov     eax,edi
                mul     eax
                sub     eax,4
                mov     ebx,eax
                mov     edx,[ebp+12]
                cmp     edx,eax
                jb      @@1780
                mov     eax,edx
                xor     edx,edx
                div     ebx
@@1780:         mov     eax,[ebp+8]
                div     ebx
                cmp     edx,0
                je      @@1030
@@1790:         bsf     ecx,edx
                shr     edx,cl
                test    ecx,1
                jz      @@1800
                mov     eax,ebx
                and     eax,7
                cmp     al,3
                je      @@1810
                cmp     al,5
                jne     @@1800
@@1810:         inc     esi
@@1800:         cmp     edx,1
                jne     @@1820
                test    esi,1
                jnz     @@1770
                jmp     @@1830
@@1820:         mov     ecx,edx
                mov     eax,ebx
                xor     edx,edx
                div     ecx
                cmp     edx,0
                je      @@1030
@@1840:         mov     eax,ebx
                and     eax,3
                cmp     al,3
                jne     @@1850
                mov     eax,ecx
                and     eax,3
                cmp     al,3
                jne     @@1850
                inc     esi
@@1850:         mov     ebx,ecx
                jmp     @@1790
@@1830:         mov     tk,edi
                mov     eax,[ebp+8]
                mov     ebx,[ebp+12]
                lea     edi,nv
                sub     eax,2
                sbb     ebx,0
                mov     [edi],eax
                mov     [edi+4],ebx
                lea     edi,nt
                mov     ecx,tk
                mov     [edi],ecx
                mov     dword ptr [edi+4],0
                add     eax,3
                adc     ebx,0
                bsf     ecx,eax
                mov     u,ecx
                cmp     ecx,32
                jb      @@1860
                shrd    eax,ebx,31
                shr     ebx,31
                sub     ecx,31
@@1860:         shrd    eax,ebx,cl
                shr     ebx,cl
                lea     esi,nu
                mov     [esi],eax
                mov     [esi+4],ebx
                cmp     ebx,0
                jne     @@1870
                cmp     eax,1
                je      @@1880
                bsr     eax,eax
                jmp     @@1890
@@1870:         bsr     eax,ebx
                add     eax,32
@@1890:         dec     eax
                mov     h,eax
                lea     edi,m
                lea     ebx,nt
                mov     eax,[ebx]
                mul     eax
                mov     [edi],eax
                cmp     dword ptr [ebx+4],0
                jne     @@1900
                mov     [edi+4],edx
                mov     dword ptr [edi+8],0
                mov     dword ptr [edi+12],0
                mov     ecx,0
                cmp     edx,0
                je      @@1910
                inc     ecx
                jmp     @@1920
@@1900:         mov     ecx,edx
                mov     eax,[ebx]
                mov     edx,[ebx+4]
                mul     edx
                mov     reg_a,eax
                mov     reg_b,edx
                add     eax,ecx
                adc     edx,0
                mov     ecx,eax
                mov     esi,edx
                add     ecx,reg_a
                adc     reg_b,0
                mov     [edi+4],ecx
                mov     ecx,reg_b
                mov     eax,[ebx+4]
                mul     eax
                add     eax,ecx
                adc     edx,0
                add     eax,esi
                adc     edx,0
                mov     [edi+8],eax
                mov     [edi+12],edx
                mov     ecx,2
                cmp     edx,0
                je      @@1920
                inc     ecx
                jmp     @@1920
                mov     ecx,3
                cmp     dword ptr [edi+12],0
                jne     @@1920
                dec     ecx
                cmp     dword ptr [edi+8],0
                jne     @@1920
                dec     ecx
                cmp     dword ptr [edi+4],0
                jne     @@1920
@@1910:         mov     ebx,[edi]
                sub     ebx,2
                lea     edi,ny
                mov     [edi],ebx
                mov     dword ptr [edi+4],0
                jmp     @@1930
@@1920:         mov     eax,2
                sub     [edi],eax
                jnc     @@1940
                sbb     dword ptr [edi+4],0
                jnc     @@1940
                sbb     dword ptr [edi+8],0
                jnc     @@1940
                sbb     dword ptr [edi+12],0
@@1940:         mov     reg_c,ecx
                mov     ebx,[ebp+8]
                mov     eax,[ebp+12]
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@1950:         cmp     edx,reg_b
                je      @@1960
                div     reg_b
                jmp     @@1970
@@1960:         add     edx,eax
                mov     eax,4294967295
                jnc     @@1970
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@1980
                mov     ebx,[edi]
@@1980:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@1990
@@1970:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2000
                mov     ebx,[edi]
@@2000:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@2010
                ja      @@2020
                test    eax,eax
                jz      @@2010
@@2020:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@2010
                ja      @@2020
                test    eax,eax
                jnz     @@2020
@@2010:         neg     eax
                adc     edx,0
                neg     edx
@@1990:         popf
                jnz     @@1950
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,ny
                mov     [edi],eax
                mov     [edi+4],edx
@@1930:         lea     edi,nu
                mov     ebx,h
                cmp     ebx,32
                jb      @@2030
                mov     eax,[edi+4]
                jmp     @@2040
@@2030:         mov     eax,[edi]
@@2040:         bt      eax,ebx
                jnc     @@2050
                lea     ebx,nt
                lea     esi,ny
                lea     edi,m
                mov     eax,[ebx]
                mov     edx,[esi]
                mul     edx
                mov     ecx,edx
                mov     [edi],eax
                cmp     dword ptr [ebx+4],0
                jne     @@2060
                mov     eax,[ebx]
                mov     edx,[esi+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     [edi+4],ecx
                mov     [edi+8],edx
                mov     dword ptr [edi+12],0
                cmp     edx,0
                je      @@2070
                mov     ecx,2
                jmp     @@2080
@@2070:         cmp     ecx,0
                je      @@2090
                mov     ecx,1
                jmp     @@2080
@@2060:         cmp     dword ptr [esi+4],0
                jne     @@2100
                mov     eax,[esi]
                mov     edx,[ebx+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     [edi+4],ecx
                mov     [edi+8],edx
                mov     dword ptr [edi+12],0
                cmp     edx,0
                je      @@2110
                mov     ecx,2
                jmp     @@2080
@@2110:         cmp     ecx,0
                je      @@2090
                mov     ecx,1
                jmp     @@2080
@@2100:         mov     eax,[esi]
                mov     edx,[ebx+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     rc,edx
                mov     eax,[ebx]
                mov     edx,[esi+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     [edi+4],ecx
                mov     ecx,edx
                mov     eax,[ebx+4]
                mov     edx,[esi+4]
                mul     edx
                add     eax,rc
                adc     edx,0
                add     eax,ecx
                adc     edx,0
                mov     [edi+8],eax
                mov     [edi+12],edx
                mov     ecx,2
                cmp     edx,0
                je      @@2080
                inc     ecx
                jmp     @@2080
                mov     ecx,3
                cmp     dword ptr [edi+12],0
                jne     @@2080
                dec     ecx
                cmp     dword ptr [edi+8],0
                jne     @@2080
                dec     ecx
                cmp     dword ptr [edi+4],0
                jne     @@2080
@@2090:         mov     eax,tk
                mov     ebx,[edi]
                lea     edi,nt
                cmp     ebx,eax
                ja      @@2120
                jne     @@2130
                mov     dword ptr [edi],0
                mov     dword ptr [edi+4],0
                jmp     @@2140
@@2130:         sub     eax,ebx
                mov     esi,[ebp+8]
                mov     ebx,[ebp+12]
                sub     esi,eax
                sbb     ebx,0
                mov     [edi],esi
                mov     [edi+4],ebx
                jmp     @@2140
@@2120:         sub     ebx,eax
                mov     [edi],ebx
                mov     dword ptr [edi+4],0
                jmp     @@2140
@@2080:         mov     eax,tk
                sub     [edi],eax
                jnc     @@2150
                sbb     dword ptr [edi+4],0
                jnc     @@2150
                sbb     dword ptr [edi+8],0
                jnc     @@2150
                sbb     dword ptr [edi+12],0
@@2150:         mov     reg_c,ecx
                mov     ebx,[ebp+8]
                mov     eax,[ebp+12]
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@2160:         cmp     edx,reg_b
                je      @@2170
                div     reg_b
                jmp     @@2180
@@2170:         add     edx,eax
                mov     eax,4294967295
                jnc     @@2180
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2190
                mov     ebx,[edi]
@@2190:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@2200
@@2180:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2210
                mov     ebx,[edi]
@@2210:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@2220
                ja      @@2230
                test    eax,eax
                jz      @@2220
@@2230:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@2220
                ja      @@2230
                test    eax,eax
                jnz     @@2230
@@2220:         neg     eax
                adc     edx,0
                neg     edx
@@2200:         popf
                jnz     @@2160
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,nt
                mov     [edi],eax
                mov     [edi+4],edx
@@2140:         lea     edi,m
                lea     ebx,ny
                mov     eax,[ebx]
                mul     eax
                mov     [edi],eax
                cmp     dword ptr [ebx+4],0
                jne     @@2240
                mov     [edi+4],edx
                mov     dword ptr [edi+8],0
                mov     dword ptr [edi+12],0
                mov     ecx,0
                cmp     edx,0
                je      @@2250
                inc     ecx
                jmp     @@2260
@@2240:         mov     ecx,edx
                mov     eax,[ebx]
                mov     edx,[ebx+4]
                mul     edx
                mov     reg_a,eax
                mov     reg_b,edx
                add     eax,ecx
                adc     edx,0
                mov     ecx,eax
                mov     esi,edx
                add     ecx,reg_a
                adc     reg_b,0
                mov     [edi+4],ecx
                mov     ecx,reg_b
                mov     eax,[ebx+4]
                mul     eax
                add     eax,ecx
                adc     edx,0
                add     eax,esi
                adc     edx,0
                mov     [edi+8],eax
                mov     [edi+12],edx
                mov     ecx,2
                cmp     edx,0
                je      @@2260
                inc     ecx
                jmp     @@2260
                mov     ecx,3
                cmp     dword ptr [edi+12],0
                jne     @@2260
                dec     ecx
                cmp     dword ptr [edi+8],0
                jne     @@2260
                dec     ecx
                cmp     dword ptr [edi+4],0
                jne     @@2260
@@2250:         mov     eax,2
                mov     ebx,[edi]
                lea     edi,ny
                cmp     ebx,eax
                ja      @@2270
                jne     @@2280
                mov     dword ptr [edi],0
                mov     dword ptr [edi+4],0
                jmp     @@2290
@@2280:         sub     eax,ebx
                mov     esi,[ebp+8]
                mov     ebx,[ebp+12]
                sub     esi,eax
                sbb     ebx,0
                mov     [edi],esi
                mov     [edi+4],ebx
                jmp     @@2290
@@2270:         sub     ebx,eax
                mov     [edi],ebx
                mov     dword ptr [edi+4],0
                jmp     @@2290
@@2260:         mov     eax,2
                sub     [edi],eax
                jnc     @@2300
                sbb     dword ptr [edi+4],0
                jnc     @@2300
                sbb     dword ptr [edi+8],0
                jnc     @@2300
                sbb     dword ptr [edi+12],0
@@2300:         mov     reg_c,ecx
                mov     ebx,[ebp+8]
                mov     eax,[ebp+12]
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@2310:         cmp     edx,reg_b
                je      @@2320
                div     reg_b
                jmp     @@2330
@@2320:         add     edx,eax
                mov     eax,4294967295
                jnc     @@2330
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2340
                mov     ebx,[edi]
@@2340:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@2350
@@2330:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2360
                mov     ebx,[edi]
@@2360:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@2370
                ja      @@2380
                test    eax,eax
                jz      @@2370
@@2380:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@2370
                ja      @@2380
                test    eax,eax
                jnz     @@2380
@@2370:         neg     eax
                adc     edx,0
                neg     edx
@@2350:         popf
                jnz     @@2310
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,ny
                mov     [edi],eax
                mov     [edi+4],edx
                dec     h
                jns     @@1930
                jmp     @@2390
@@2050:         lea     ebx,nt
                lea     esi,ny
                lea     edi,m
                mov     eax,[ebx]
                mov     edx,[esi]
                mul     edx
                mov     ecx,edx
                mov     [edi],eax
                cmp     dword ptr [ebx+4],0
                jne     @@2400
                mov     eax,[ebx]
                mov     edx,[esi+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     [edi+4],ecx
                mov     [edi+8],edx
                mov     dword ptr [edi+12],0
                cmp     edx,0
                je      @@2410
                mov     ecx,2
                jmp     @@2420
@@2410:         cmp     ecx,0
                je      @@2430
                mov     ecx,1
                jmp     @@2420
@@2400:         cmp     dword ptr [esi+4],0
                jne     @@2440
                mov     eax,[esi]
                mov     edx,[ebx+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     [edi+4],ecx
                mov     [edi+8],edx
                mov     dword ptr [edi+12],0
                cmp     edx,0
                je      @@2450
                mov     ecx,2
                jmp     @@2420
@@2450:         cmp     ecx,0
                je      @@2430
                mov     ecx,1
                jmp     @@2420
@@2440:         mov     eax,[esi]
                mov     edx,[ebx+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     rc,edx
                mov     eax,[ebx]
                mov     edx,[esi+4]
                mul     edx
                add     ecx,eax
                adc     edx,0
                mov     [edi+4],ecx
                mov     ecx,edx
                mov     eax,[ebx+4]
                mov     edx,[esi+4]
                mul     edx
                add     eax,rc
                adc     edx,0
                add     eax,ecx
                adc     edx,0
                mov     [edi+8],eax
                mov     [edi+12],edx
                mov     ecx,2
                cmp     edx,0
                je      @@2420
                inc     ecx
                jmp     @@2420
                mov     ecx,3
                cmp     dword ptr [edi+12],0
                jne     @@2420
                dec     ecx
                cmp     dword ptr [edi+8],0
                jne     @@2420
                dec     ecx
                cmp     dword ptr [edi+4],0
                jne     @@2420
@@2430:         mov     eax,tk
                mov     ebx,[edi]
                lea     edi,ny
                cmp     ebx,eax
                ja      @@2460
                jne     @@2470
                mov     dword ptr [edi],0
                mov     dword ptr [edi+4],0
                jmp     @@2480
@@2470:         sub     eax,ebx
                mov     esi,[ebp+8]
                mov     ebx,[ebp+12]
                sub     esi,eax
                sbb     ebx,0
                mov     [edi],esi
                mov     [edi+4],ebx
                jmp     @@2480
@@2460:         sub     ebx,eax
                mov     [edi],ebx
                mov     dword ptr [edi+4],0
                jmp     @@2480
@@2420:         mov     eax,tk
                sub     [edi],eax
                jnc     @@2490
                sbb     dword ptr [edi+4],0
                jnc     @@2490
                sbb     dword ptr [edi+8],0
                jnc     @@2490
                sbb     dword ptr [edi+12],0
@@2490:         mov     reg_c,ecx
                mov     ebx,[ebp+8]
                mov     eax,[ebp+12]
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@2500:         cmp     edx,reg_b
                je      @@2510
                div     reg_b
                jmp     @@2520
@@2510:         add     edx,eax
                mov     eax,4294967295
                jnc     @@2520
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2530
                mov     ebx,[edi]
@@2530:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@2540
@@2520:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2550
                mov     ebx,[edi]
@@2550:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@2560
                ja      @@2570
                test    eax,eax
                jz      @@2560
@@2570:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@2560
                ja      @@2570
                test    eax,eax
                jnz     @@2570
@@2560:         neg     eax
                adc     edx,0
                neg     edx
@@2540:         popf
                jnz     @@2500
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,ny
                mov     [edi],eax
                mov     [edi+4],edx
@@2480:         lea     edi,m
                lea     ebx,nt
                mov     eax,[ebx]
                mul     eax
                mov     [edi],eax
                cmp     dword ptr [ebx+4],0
                jne     @@2580
                mov     [edi+4],edx
                mov     dword ptr [edi+8],0
                mov     dword ptr [edi+12],0
                mov     ecx,0
                cmp     edx,0
                je      @@2590
                inc     ecx
                jmp     @@2600
@@2580:         mov     ecx,edx
                mov     eax,[ebx]
                mov     edx,[ebx+4]
                mul     edx
                mov     reg_a,eax
                mov     reg_b,edx
                add     eax,ecx
                adc     edx,0
                mov     ecx,eax
                mov     esi,edx
                add     ecx,reg_a
                adc     reg_b,0
                mov     [edi+4],ecx
                mov     ecx,reg_b
                mov     eax,[ebx+4]
                mul     eax
                add     eax,ecx
                adc     edx,0
                add     eax,esi
                adc     edx,0
                mov     [edi+8],eax
                mov     [edi+12],edx
                mov     ecx,2
                cmp     edx,0
                je      @@2600
                inc     ecx
                jmp     @@2600
                mov     ecx,3
                cmp     dword ptr [edi+12],0
                jne     @@2600
                dec     ecx
                cmp     dword ptr [edi+8],0
                jne     @@2600
                dec     ecx
                cmp     dword ptr [edi+4],0
                jne     @@2600
@@2590:         mov     eax,2
                mov     ebx,[edi]
                lea     edi,nt
                cmp     ebx,eax
                ja      @@2610
                jne     @@2620
                mov     dword ptr [edi],0
                mov     dword ptr [edi+4],0
                jmp     @@2290
@@2620:         sub     eax,ebx
                mov     esi,[ebp+8]
                mov     ebx,[ebp+12]
                sub     esi,eax
                sbb     ebx,0
                mov     [edi],esi
                mov     [edi+4],ebx
                jmp     @@2290
@@2610:         sub     ebx,eax
                mov     [edi],ebx
                mov     dword ptr [edi+4],0
                jmp     @@2290
@@2600:         mov     eax,2
                sub     [edi],eax
                jnc     @@2630
                sbb     dword ptr [edi+4],0
                jnc     @@2630
                sbb     dword ptr [edi+8],0
                jnc     @@2630
                sbb     dword ptr [edi+12],0
@@2630:         mov     reg_c,ecx
                mov     ebx,[ebp+8]
                mov     eax,[ebp+12]
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@2640:         cmp     edx,reg_b
                je      @@2650
                div     reg_b
                jmp     @@2660
@@2650:         add     edx,eax
                mov     eax,4294967295
                jnc     @@2660
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2670
                mov     ebx,[edi]
@@2670:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@2680
@@2660:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2690
                mov     ebx,[edi]
@@2690:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@2700
                ja      @@2710
                test    eax,eax
                jz      @@2700
@@2710:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@2700
                ja      @@2710
                test    eax,eax
                jnz     @@2710
@@2700:         neg     eax
                adc     edx,0
                neg     edx
@@2680:         popf
                jnz     @@2640
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,nt
                mov     [edi],eax
                mov     [edi+4],edx
@@2290:         dec     h
                jns     @@1930
@@2390:         lea     edi,nt
                mov     eax,[edi]
                mov     ebx,[edi+4]
                cmp     eax,2
                jne     @@2720
                cmp     ebx,0
                jne     @@2720
                mov     eax,1
                jmp     @@1020
@@2720:         lea     esi,nv
                cmp     eax,[esi]
                jne     @@1880
                cmp     ebx,[esi+4]
                jne     @@1880
                mov     eax,1
                jmp     @@1020
@@1880:         dec     u
                jz      @@1030
                neg     u
@@2730:         lea     edi,m
                lea     ebx,nt
                mov     eax,[ebx]
                mul     eax
                mov     [edi],eax
                cmp     dword ptr [ebx+4],0
                jne     @@2740
                mov     [edi+4],edx
                mov     dword ptr [edi+8],0
                mov     dword ptr [edi+12],0
                mov     ecx,0
                cmp     edx,0
                je      @@2750
                inc     ecx
                jmp     @@2760
@@2740:         mov     ecx,edx
                mov     eax,[ebx]
                mov     edx,[ebx+4]
                mul     edx
                mov     reg_a,eax
                mov     reg_b,edx
                add     eax,ecx
                adc     edx,0
                mov     ecx,eax
                mov     esi,edx
                add     ecx,reg_a
                adc     reg_b,0
                mov     [edi+4],ecx
                mov     ecx,reg_b
                mov     eax,[ebx+4]
                mul     eax
                add     eax,ecx
                adc     edx,0
                add     eax,esi
                adc     edx,0
                mov     [edi+8],eax
                mov     [edi+12],edx
                mov     ecx,2
                cmp     edx,0
                je      @@2760
                inc     ecx
                jmp     @@2760
                mov     ecx,3
                cmp     dword ptr [edi+12],0
                jne     @@2760
                dec     ecx
                cmp     dword ptr [edi+8],0
                jne     @@2760
                dec     ecx
                cmp     dword ptr [edi+4],0
                jne     @@2760
@@2750:         mov     eax,2
                mov     ebx,[edi]
                lea     edi,nt
                cmp     ebx,eax
                ja      @@2770
                jne     @@2780
                mov     dword ptr [edi],0
                mov     dword ptr [edi+4],0
                jmp     @@2790
@@2780:         sub     eax,ebx
                mov     esi,[ebp+8]
                mov     ebx,[ebp+12]
                sub     esi,eax
                sbb     ebx,0
                mov     [edi],esi
                mov     [edi+4],ebx
                jmp     @@2790
@@2770:         sub     ebx,eax
                mov     [edi],ebx
                mov     dword ptr [edi+4],0
                jmp     @@2790
@@2760:         mov     eax,2
                sub     [edi],eax
                jnc     @@2800
                sbb     dword ptr [edi+4],0
                jnc     @@2800
                sbb     dword ptr [edi+8],0
                jnc     @@2800
                sbb     dword ptr [edi+12],0
@@2800:         mov     reg_c,ecx
                mov     ebx,[ebp+8]
                mov     eax,[ebp+12]
                lea     edi,[edi+ecx*4]
                bsr     ecx,eax
                neg     ecx
                add     ecx,31
                shld    eax,ebx,cl
                shl     ebx,cl
                mov     reg_b,eax
                mov     reg_a,ebx
                xor     edx,edx
                mov     eax,[edi]
                mov     esi,edx
                sub     edi,4
                mov     ebx,[edi]
                shld    edx,eax,cl
                shld    eax,ebx,cl
@@2810:         cmp     edx,reg_b
                je      @@2820
                div     reg_b
                jmp     @@2830
@@2820:         add     edx,eax
                mov     eax,4294967295
                jnc     @@2830
                sub     edi,4
                mov     esi,eax
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2840
                mov     ebx,[edi]
@@2840:         add     eax,reg_a
                adc     edx,0
                sub     edx,reg_a
                jmp     @@2850
@@2830:         mov     esi,eax
                sub     edi,4
                mov     eax,ebx
                mov     ebx,reg_c
                dec     ebx
                mov     reg_c,ebx
                pushf
                jz      @@2860
                mov     ebx,[edi]
@@2860:         shld    eax,ebx,cl
                mov     reg_s,edx
                mov     reg_u,eax
                mov     eax,reg_a
                mul     esi
                sub     eax,reg_u
                sbb     edx,reg_s
                jb      @@2870
                ja      @@2880
                test    eax,eax
                jz      @@2870
@@2880:         dec     esi
                sub     eax,reg_a
                sbb     edx,reg_b
                jb      @@2870
                ja      @@2880
                test    eax,eax
                jnz     @@2880
@@2870:         neg     eax
                adc     edx,0
                neg     edx
@@2850:         popf
                jnz     @@2810
                shrd    eax,edx,cl
                shr     edx,cl
                lea     edi,nt
                mov     [edi],eax
                mov     [edi+4],edx
@@2790:         lea     edi,nt
                mov     eax,[edi]
                mov     ebx,[edi+4]
                lea     esi,nv
                cmp     eax,[esi]
                jne     @@2890
                cmp     ebx,[esi+4]
                jne     @@2890
                mov     eax,1
                jmp     @@1020
@@2890:         cmp     eax,2
                jne     @@2900
                cmp     ebx,0
                je      @@1030
@@2900:         inc     u
                jnz     @@2730
@@1030:         xor     eax,eax
@@1020:
END;

END.
